@@ -25,7 +25,7 @@
 #' option return NA for alleles not in bin. The calculate option handles
 #' all named alleles including micro variants (e.g. '9.3').
 #' Handles 'X' and 'Y' by replacing them with '1' and '2'.
-#' 
+#'
 #' @param data data.frame with at least columns 'Marker' and 'Allele'.
 #' @param kit data.frame with columns 'Marker', 'Allele', and 'Size' (for bins=TRUE) or
 #'  'Marker', 'Allele', 'Offset' and 'Repeat' (for bins=FALSE).
@@ -33,17 +33,16 @@
 #'  If FALSE the size is calculated from the locus offset and repeat unit.
 #' @param ignore.case logical TRUE case in marker names are ignored.
 #' @param debug logical indicating printing debug information.
-#' 
+#'
 #' @return data.frame with additional columns for added size.
-#' 
+#'
 #' @export
-#' 
+#'
 #' @importFrom utils head str
-#' 
+#'
 
-addSize <- function(data, kit=NA, bins=TRUE, ignore.case=FALSE, debug=FALSE){
-  
-  if(debug){
+addSize <- function(data, kit = NA, bins = TRUE, ignore.case = FALSE, debug = FALSE) {
+  if (debug) {
     print(paste("IN:", match.call()[[1]]))
     print("PARAMETERS:")
     print("data:")
@@ -53,106 +52,104 @@ addSize <- function(data, kit=NA, bins=TRUE, ignore.case=FALSE, debug=FALSE){
     print(str(kit))
     print(head(kit))
   }
-  
+
   # CHECK DATA ----------------------------------------------------------------
-  
+
   # Check dataset.
-  if(!"Marker" %in% names(data)){
+  if (!"Marker" %in% names(data)) {
     stop("'data' must contain a column 'Marker'",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
-  
-  if(!"Allele" %in% names(data)){
+
+  if (!"Allele" %in% names(data)) {
     stop("'data' must contain a column 'Allele'",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
-  
+
   # Check kit depending on 'bins'.
-  if(bins){
-    
-    if(!"Size" %in% names(kit)){
+  if (bins) {
+    if (!"Size" %in% names(kit)) {
       stop("'kit' must contain a column 'Size'",
-           call. = TRUE)
+        call. = TRUE
+      )
     }
-    
-    if(!"Allele" %in% names(kit)){
+
+    if (!"Allele" %in% names(kit)) {
       stop("'kit' must contain a column 'Allele'",
-           call. = TRUE)
+        call. = TRUE
+      )
     }
-    
   } else {
-    
-    if(!"Offset" %in% names(kit)){
+    if (!"Offset" %in% names(kit)) {
       stop("'kit' must contain a column 'Offset'",
-           call. = TRUE)
+        call. = TRUE
+      )
     }
-    
-    if(!"Repeat" %in% names(kit)){
+
+    if (!"Repeat" %in% names(kit)) {
       stop("'kit' must contain a column 'Repeat'",
-           call. = TRUE)
+        call. = TRUE
+      )
     }
-    
   }
-  
+
   # Check kit.
-  if(!"Marker" %in% names(kit)){
+  if (!"Marker" %in% names(kit)) {
     stop("'kit' must contain a column 'Marker'",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
-  
+
   # Check if character data.
-  if(!is.character(data$Allele)){
+  if (!is.character(data$Allele)) {
     message("'Allele' must be character. 'data' converted")
     data$Allele <- as.character(data$Allele)
   }
-  
+
   # PREPARE -----------------------------------------------------------------
-  
+
   # Check for column 'Size'
-  if("Size" %in% names(data)){
-    
+  if ("Size" %in% names(data)) {
     message("'data' already contain a column 'Size'")
     message("Size will be overwritten!")
-    
   }
-  
+
   # Add a column 'Size'
   data$Size <- NA
 
   # Get markers in dataset.
   marker <- unique(data$Marker)
-  
+
   # Check if case in marker names should be ignored.
-  if(ignore.case){
+  if (ignore.case) {
     kit$Marker <- toupper(kit$Marker)
     marker <- toupper(marker)
   }
-  
+
   # ADD SIZE ------------------------------------------------------------------
 
-  if(debug){
+  if (debug) {
     print("Markers:")
     print(marker)
   }
 
   # Loop over markers.
-  for(m in seq(along=marker)){
-    
-    if(debug){
+  for (m in seq(along = marker)) {
+    if (debug) {
       print("Current marker:")
       print(marker[m])
     }
-    
-    if(ignore.case){
+
+    if (ignore.case) {
 
       # Select rows for current marker and ignore case in marker names.
       cMarker <- toupper(data$Marker) == toupper(marker[m])
-      
     } else {
-      
+
       # Select rows for current marker.
       cMarker <- data$Marker == marker[m]
-      
     }
 
     # Get alleles for current marker.
@@ -160,77 +157,70 @@ addSize <- function(data, kit=NA, bins=TRUE, ignore.case=FALSE, debug=FALSE){
     # Remove NAs.
     allele <- allele[!is.na(allele)]
 
-    if(debug){
+    if (debug) {
       print("Alleles:")
       print(allele)
     }
-    
+
     # Loop over allele in current marker.
-    for(a in seq(along=allele)){
-      
+    for (a in seq(along = allele)) {
+
       # Select rows for current allele.
       cAllele <- data$Allele == allele[a]
-      
+
       # Combine selections.
       selection <- cMarker & cAllele
-      
-      if(bins){
-        
+
+      if (bins) {
+
         # Get size from matching bins.
         size <- kit$Size[kit$Marker == marker[m] & kit$Allele == allele[a]]
-        
       } else {
         # Calculate size from 'offset' and 'repeat'.
-        
+
         # Copy to temporary working variable.
         alleleTmp <- toupper(allele[a])
-        
+
         # Check presence of X/Y.
         if ("X" %in% alleleTmp || "Y" %in% alleleTmp) {
-          
+
           # Use 1 and 2 for X and Y.
-          alleleTmp <- sub(pattern="X", replacement=1, x=alleleTmp)
-          alleleTmp <- sub(pattern="Y", replacement=2, x=alleleTmp)
+          alleleTmp <- sub(pattern = "X", replacement = 1, x = alleleTmp)
+          alleleTmp <- sub(pattern = "Y", replacement = 2, x = alleleTmp)
         }
-        
+
         # Convert to numeric.
         alleleTmp <- as.numeric(alleleTmp)
-        
+
         # Calculate estimated size.
         tmpOffset <- kit$Offset[kit$Marker == marker[m]]
         tmpRepeat <- kit$Repeat[kit$Marker == marker[m]]
         size <- tmpOffset + floor(alleleTmp) * tmpRepeat + (alleleTmp %% 1) * 10
-        
       }
-      
+
       # Store size for current allele in current marker.
-      if(length(size) != 0){
-        
+      if (length(size) != 0) {
         data$Size[selection] <- size
-        
       } else {
-        
-        message(paste("Allele", allele[a],
-                      "for marker", marker[m], "not in kit definition file."))
-        
+        message(paste(
+          "Allele", allele[a],
+          "for marker", marker[m], "not in kit definition file."
+        ))
       }
-      
     }
-    
   }
-  
-  if(debug){
+
+  if (debug) {
     print("Return:")
     print(str(data))
     print(head(data))
   }
-  
+
   # Add attributes to result.
-  attr(data, which="kit") <- kit
-  
+  attr(data, which = "kit") <- kit
+
   # Update audit trail.
   data <- auditTrail(obj = data, f.call = match.call(), package = "strvalidator")
-  
+
   return(data)
-  
 }
