@@ -1,5 +1,7 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 03.03.2020: Fixed reference to function name.
+# 25.02.2020: Added language support.
 # 03.03.2019: Compacted and tweaked widgets under tcltk.
 # 17.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 06.08.2017: Added audit trail.
@@ -46,13 +48,89 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
   .gDataName <- NULL
   .gKit <- 1
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Add size to dataset"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset and kit"
+  strLblDataset <- "Sample dataset:"
+  strDrpDefault <- "<Select dataset>"
+  strLblSamples <- "samples"
+  strLblKit <- "Kit:"
+  strFrmOptions <- "Options"
+  strRadItem1 <- "Get size as defined in bins file (NA if not defined)"
+  strRadItem2 <- "Calculate an estimate from locus offset and number of repeats"
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnAdd <- "Add"
+  strBtnProcessing <- "Processing..."
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDefault"]$value
+    strDrpDefault <- ifelse(is.na(strtmp), strDrpDefault, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strLblKit"]$value
+    strLblKit <- ifelse(is.na(strtmp), strLblKit, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strRadItem1"]$value
+    strRadItem1 <- ifelse(is.na(strtmp), strRadItem1, strtmp)
+
+    strtmp <- dtStrings["strRadItem2"]$value
+    strRadItem2 <- ifelse(is.na(strtmp), strRadItem2, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnAdd"]$value
+    strBtnAdd <- ifelse(is.na(strtmp), strBtnAdd, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+  }
+
+  # ---------------------------------------------------------------------------
 
   # Main window.
-  w <- gwindow(title = "Add size to dataset", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -96,22 +174,22 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("addSize_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # DATASET ###################################################################
 
   f0 <- gframe(
-    text = "Dataset and kit",
+    text = strFrmDataset,
     horizontal = FALSE,
     spacing = 2,
     container = gv
@@ -119,11 +197,11 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
 
   f0g1 <- glayout(container = f0, spacing = 1)
 
-  f0g1[1, 1] <- glabel(text = "Select dataset:", container = f0g1)
+  f0g1[1, 1] <- glabel(text = strLblDataset, container = f0g1)
 
   f0g1[1, 2] <- dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDefault,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -136,7 +214,7 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
   )
 
   f0g1[1, 3] <- dataset_samples_lbl <- glabel(
-    text = " 0 samples",
+    text = paste(" 0", strLblSamples),
     container = f0g1
   )
 
@@ -156,28 +234,23 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
       .gData <<- get(val_obj, envir = env)
       .gDataName <<- val_obj
       samples <- length(unique(.gData$Sample.Name))
-      svalue(dataset_samples_lbl) <- paste(" ", samples, "samples")
+      svalue(dataset_samples_lbl) <- paste(" ", samples, strLblSamples)
       .gKit <<- detectKit(.gData, index = TRUE)
       svalue(kit_drp, index = TRUE) <- .gKit
       svalue(save_edt) <- paste(.gDataName, "_size", sep = "")
-
-      if (debug) {
-        print("Detected kit index")
-        print(.gKit)
-      }
     } else {
 
       # Reset components.
       .gData <<- data.frame(No.Data = NA)
       .gDataName <<- NULL
-      svalue(dataset_samples_lbl) <- " 0 samples"
+      svalue(dataset_samples_lbl) <- paste(" 0", strLblSamples)
       svalue(save_edt) <- ""
     }
   })
 
   # KIT -----------------------------------------------------------------------
 
-  f0g1[2, 1] <- glabel(text = "Kit:", container = f0g1)
+  f0g1[2, 1] <- glabel(text = strLblKit, container = f0g1)
 
   kit_drp <- gcombobox(
     items = getKit(),
@@ -194,34 +267,27 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
   # OPTIONS -----------------------------------------------------------------------
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 2,
     container = gv
   )
 
-  f1_items <- c(
-    "Get size as defined in bins file (NA if not defined)",
-    "Calculate an estimate from locus offset and number of repeats"
-  )
+  f1_items <- c(strRadItem1, strRadItem2)
 
   f1_size_opt <- gradio(items = f1_items, selected = 2, container = f1)
 
   # SAVE ######################################################################
 
-  save_frame <- gframe(text = "Save as", container = gv)
+  save_frame <- gframe(text = strFrmSave, container = gv)
 
-  glabel(text = "Name for result:", container = save_frame)
+  glabel(text = strLblSave, container = save_frame)
 
   save_edt <- gedit(expand = TRUE, fill = TRUE, container = save_frame)
 
   # BUTTON ####################################################################
 
-  if (debug) {
-    print("BUTTON")
-  }
-
-  add_btn <- gbutton(text = "Add size", container = gv)
+  add_btn <- gbutton(text = strBtnAdd, container = gv)
 
   addHandlerClicked(add_btn, handler = function(h, ...) {
 
@@ -252,7 +318,7 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
 
     # Change button.
     blockHandlers(add_btn)
-    svalue(add_btn) <- "Processing..."
+    svalue(add_btn) <- strBtnProcessing
     unblockHandlers(add_btn)
     enabled(add_btn) <- FALSE
 
@@ -272,7 +338,7 @@ addSize_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, par
     # Update audit trail.
     datanew <- auditTrail(
       obj = datanew, key = keys, value = values,
-      label = "addSize_gui", arguments = FALSE,
+      label = fnc, arguments = FALSE,
       package = "strvalidator"
     )
 
