@@ -1,5 +1,7 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 03.03.2020: Fixed reference to function name.
+# 02.03.2020: Added language support.
 # 03.03.2019: Compacted and tweaked widgets under tcltk.
 # 17.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 06.08.2017: Added audit trail.
@@ -18,8 +20,6 @@
 # 26.09.2014: Implemented text field for 'exclude'.
 # 12.09.2014: Implemented new options 'exclude OL'.
 # 28.06.2014: Added help button and moved save gui checkbox.
-# 25.02.2014: Implemented new options 'replace NA' and 'add to dataset'.
-# 18.07.2013: Check before overwrite object.
 
 #' @title Calculate Peak Height
 #'
@@ -59,12 +59,157 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
   .gRef <- NULL
   .gRefName <- NULL
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Calculate peak height"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Datasets"
+  strLblDataset <- "Sample dataset:"
+  strDrpDefault <- "<Select dataset>"
+  strLblSamples <- "samples"
+  strLblRefDataset <- "Reference dataset:"
+  strLblRef <- "references"
+  strBtnCheck <- "Check subsetting"
+  strLblKit <- "Kit:"
+  strFrmOptions <- "Options"
+  strLblPre <- "Pre-processing:"
+  strChkSex <- "Remove sex markers"
+  strChkSensors <- "Remove quality sensors"
+  strChkExclude <- "Exclude values in 'Allele' column"
+  strChkExcludeInfo <- "Case sensitive values separated by comma:"
+  strLblMatching <- "Reference sample name matching:"
+  strChkIgnore <- "Ignore case"
+  strChkExact <- "Exact matching"
+  strLblPost <- "Post-processing:"
+  strChkReplace <- "Replace NA in the result with 0"
+  strChkAdd <- "Add result to dataset"
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnCalculate <- "Calculate"
+  strBtnProcessing <- "Processing..."
+  strMsgDataset <- "A sample dataset and a reference dataset must be selected."
+  strMsgTitleDataset <- "Dataset not selected"
+  strMsgCheck <- "Data frame is NULL!\n\nMake sure to select a sample dataset and a reference dataset."
+  strWinTitleCheck <- "Check subsetting"
+  strMsgTitleError <- "Error"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDefault"]$value
+    strDrpDefault <- ifelse(is.na(strtmp), strDrpDefault, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strLblRefDataset"]$value
+    strLblRefDataset <- ifelse(is.na(strtmp), strLblRefDataset, strtmp)
+
+    strtmp <- dtStrings["strLblRef"]$value
+    strLblRef <- ifelse(is.na(strtmp), strLblRef, strtmp)
+
+    strtmp <- dtStrings["strBtnCheck"]$value
+    strBtnCheck <- ifelse(is.na(strtmp), strBtnCheck, strtmp)
+
+    strtmp <- dtStrings["strLblKit"]$value
+    strLblKit <- ifelse(is.na(strtmp), strLblKit, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strLblPre"]$value
+    strLblPre <- ifelse(is.na(strtmp), strLblPre, strtmp)
+
+    strtmp <- dtStrings["strChkSex"]$value
+    strChkSex <- ifelse(is.na(strtmp), strChkSex, strtmp)
+
+    strtmp <- dtStrings["strChkSensors"]$value
+    strChkSensors <- ifelse(is.na(strtmp), strChkSensors, strtmp)
+
+    strtmp <- dtStrings["strChkExclude"]$value
+    strChkExclude <- ifelse(is.na(strtmp), strChkExclude, strtmp)
+
+    strtmp <- dtStrings["strChkExcludeInfo"]$value
+    strChkExcludeInfo <- ifelse(is.na(strtmp), strChkExcludeInfo, strtmp)
+
+    strtmp <- dtStrings["strLblMatching"]$value
+    strLblMatching <- ifelse(is.na(strtmp), strLblMatching, strtmp)
+
+    strtmp <- dtStrings["strChkIgnore"]$value
+    strChkIgnore <- ifelse(is.na(strtmp), strChkIgnore, strtmp)
+
+    strtmp <- dtStrings["strChkExact"]$value
+    strChkExact <- ifelse(is.na(strtmp), strChkExact, strtmp)
+
+    strtmp <- dtStrings["strLblPost"]$value
+    strLblPost <- ifelse(is.na(strtmp), strLblPost, strtmp)
+
+    strtmp <- dtStrings["strChkReplace"]$value
+    strChkReplace <- ifelse(is.na(strtmp), strChkReplace, strtmp)
+
+    strtmp <- dtStrings["strChkAdd"]$value
+    strChkAdd <- ifelse(is.na(strtmp), strChkAdd, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnCalculate"]$value
+    strBtnCalculate <- ifelse(is.na(strtmp), strBtnCalculate, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strMsgDataset"]$value
+    strMsgDataset <- ifelse(is.na(strtmp), strMsgDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleDataset"]$value
+    strMsgTitleDataset <- ifelse(is.na(strtmp), strMsgTitleDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgCheck"]$value
+    strMsgCheck <- ifelse(is.na(strtmp), strMsgCheck, strtmp)
+
+    strtmp <- dtStrings["strWinTitleCheck"]$value
+    strWinTitleCheck <- ifelse(is.na(strtmp), strWinTitleCheck, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Calculate peak height", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -108,22 +253,22 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("calculateHeight_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Datasets",
+    text = strFrmDataset,
     horizontal = FALSE,
     spacing = 2,
     container = gv
@@ -133,11 +278,11 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
 
   # Datasets ------------------------------------------------------------------
 
-  f0g0[1, 1] <- glabel(text = "Select dataset:", container = f0g0)
+  f0g0[1, 1] <- glabel(text = strLblDataset, container = f0g0)
 
   f0g0[1, 2] <- dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDefault,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -148,7 +293,10 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
     ellipsize = "none"
   )
 
-  f0g0[1, 3] <- f0g0_samples_lbl <- glabel(text = " 0 samples", container = f0g0)
+  f0g0[1, 3] <- f0g0_samples_lbl <- glabel(
+    text = paste(" 0", strLblSamples),
+    container = f0g0
+  )
 
   addHandlerChanged(dataset_drp, handler = function(h, ...) {
     val_obj <- svalue(dataset_drp)
@@ -167,7 +315,7 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
       .gData <<- get(val_obj, envir = env)
       .gDataName <<- val_obj
       samples <- length(unique(.gData$Sample.Name))
-      svalue(f0g0_samples_lbl) <- paste("", samples, "samples")
+      svalue(f0g0_samples_lbl) <- paste("", samples, strLblSamples)
 
       # Suggest name for the result.
       svalue(save_edt) <- paste(val_obj, "_height", sep = "")
@@ -182,16 +330,16 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
       .gData <<- NULL
       .gDataName <<- NULL
       svalue(dataset_drp, index = TRUE) <- 1
-      svalue(f0g0_samples_lbl) <- " 0 samples"
+      svalue(f0g0_samples_lbl) <- paste(" 0", strLblSamples)
       svalue(save_edt) <- ""
     }
   })
 
-  f0g0[2, 1] <- glabel(text = "Select reference:", container = f0g0)
+  f0g0[2, 1] <- glabel(text = strLblRefDataset, container = f0g0)
 
   f0g0[2, 2] <- refset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDefault,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -202,7 +350,10 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
     ellipsize = "none"
   )
 
-  f0g0[2, 3] <- f0g0_ref_lbl <- glabel(text = " 0 references", container = f0g0)
+  f0g0[2, 3] <- f0g0_ref_lbl <- glabel(
+    text = paste(" 0", strLblRef),
+    container = f0g0
+  )
 
   addHandlerChanged(refset_drp, handler = function(h, ...) {
     val_obj <- svalue(refset_drp)
@@ -221,19 +372,19 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
       .gRef <<- get(val_obj, envir = env)
       .gRefName <<- val_obj
       ref <- length(unique(.gRef$Sample.Name))
-      svalue(f0g0_ref_lbl) <- paste("", ref, "references")
+      svalue(f0g0_ref_lbl) <- paste("", ref, strLblRef)
     } else {
 
       # Reset components.
       .gRef <<- NULL
       svalue(refset_drp, index = TRUE) <- 1
-      svalue(f0g0_ref_lbl) <- " 0 references"
+      svalue(f0g0_ref_lbl) <- paste(" 0", strLblRef)
     }
   })
 
   # CHECK ---------------------------------------------------------------------
 
-  f0g0[3, 2] <- check_btn <- gbutton(text = "Check subsetting", container = f0g0)
+  f0g0[3, 2] <- check_btn <- gbutton(text = strBtnCheck, container = f0g0)
 
   addHandlerChanged(check_btn, handler = function(h, ...) {
 
@@ -246,7 +397,7 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
 
     if (!is.null(.gData) || !is.null(.gRef)) {
       chksubset_w <- gwindow(
-        title = "Check subsetting",
+        title = strWinTitleCheck,
         visible = FALSE, name = title,
         width = NULL, height = NULL, parent = w,
         handler = NULL, action = NULL
@@ -269,9 +420,8 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
       visible(chksubset_w) <- TRUE
     } else {
       gmessage(
-        msg = "Data frame is NULL!\n\n
-               Make sure to select a dataset and a reference set",
-        title = "Error",
+        msg = strMsgCheck,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
@@ -279,7 +429,7 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
 
   # Kit -----------------------------------------------------------------------
 
-  f0g0[4, 1] <- glabel(text = "Select the kit used:", container = f0g0)
+  f0g0[4, 1] <- glabel(text = strLblKit, container = f0g0)
 
   f0g0[4, 2] <- kit_drp <- gcombobox(
     items = getKit(), selected = 1,
@@ -290,57 +440,57 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Pre-processing:", anchor = c(-1, 0), container = f1)
+  glabel(text = strLblPre, anchor = c(-1, 0), container = f1)
 
   f1_sex_chk <- gcheckbox(
-    text = "Remove sex markers",
+    text = strChkSex,
     checked = FALSE, container = f1
   )
 
   f1_qs_chk <- gcheckbox(
-    text = "Remove quality sensors",
+    text = strChkSensors,
     checked = TRUE, container = f1
   )
 
   f1_exclude_chk <- gcheckbox(
-    text = "Exclude values in 'Allele' column",
+    text = strChkExclude,
     checked = TRUE, container = f1
   )
 
   f1_exclude_lbl <- glabel(
-    text = "Case sensitive values separated by comma:",
+    text = strChkExcludeInfo,
     anchor = c(-1, 0), container = f1
   )
 
   f1_exclude_edt <- gedit(text = "OL", container = f1)
 
-  glabel(text = "Reference sample name matching:", anchor = c(-1, 0), container = f1)
+  glabel(text = strLblMatching, anchor = c(-1, 0), container = f1)
 
   f1_ignore_chk <- gcheckbox(
-    text = "Ignore case in sample name matching",
+    text = strChkIgnore,
     checked = TRUE, container = f1
   )
 
   f1_exact_chk <- gcheckbox(
-    text = "Exact sample name matching",
+    text = strChkExact,
     checked = FALSE, container = f1
   )
 
-  glabel(text = "Post-processing:", anchor = c(-1, 0), container = f1)
+  glabel(text = strLblPost, anchor = c(-1, 0), container = f1)
 
   f1_replace_chk <- gcheckbox(
-    text = "Replace NA in the result with 0",
+    text = strChkReplace,
     checked = TRUE, container = f1
   )
 
   f1_add_chk <- gcheckbox(
-    text = "Add result to dataset",
+    text = strChkAdd,
     checked = TRUE, container = f1
   )
 
@@ -356,16 +506,16 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
 
   # SAVE ######################################################################
 
-  save_frame <- gframe(text = "Save as", container = gv)
+  save_frame <- gframe(text = strFrmSave, container = gv)
 
-  glabel(text = "Name for result:", container = save_frame)
+  glabel(text = strLblSave, container = save_frame)
 
   save_edt <- gedit(expand = TRUE, fill = TRUE, container = save_frame)
 
   # BUTTON ####################################################################
 
 
-  calculate_btn <- gbutton(text = "Calculate", container = gv)
+  calculate_btn <- gbutton(text = strBtnCalculate, container = gv)
 
   addHandlerClicked(calculate_btn, handler = function(h, ...) {
     val_data <- .gData
@@ -401,7 +551,7 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
 
       # Change button.
       blockHandlers(calculate_btn)
-      svalue(calculate_btn) <- "Processing..."
+      svalue(calculate_btn) <- strBtnProcessing
       unblockHandlers(calculate_btn)
       enabled(calculate_btn) <- FALSE
 
@@ -430,7 +580,7 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
       # Update audit trail.
       datanew <- auditTrail(
         obj = datanew, key = keys, value = values,
-        label = "calculateHeight_gui", arguments = FALSE,
+        label = fnc, arguments = FALSE,
         package = "strvalidator"
       )
 
@@ -440,17 +590,16 @@ calculateHeight_gui <- function(env = parent.frame(), savegui = NULL, debug = FA
 
       if (debug) {
         print(datanew)
-        print(paste("EXIT:", match.call()[[1]]))
+        print(paste("EXIT:", fnc))
       }
 
       # Close GUI.
       .saveSettings()
       dispose(w)
     } else {
-      message <- "A dataset and a reference dataset have to be selected."
-
-      gmessage(message,
-        title = "Datasets not selected",
+      gmessage(
+        msg = strMsgDataset,
+        title = strMsgTitleDataset,
         icon = "error",
         parent = w
       )
