@@ -1,5 +1,7 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 03.03.2020: Added language support.
+# 03.03.2020: Expand scrollable checkbox view.
 # 19.02.2019: Expand text field under tcltk. Scrollable checkbox view.
 # 17.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 06.08.2017: Added audit trail.
@@ -41,12 +43,87 @@
 #' @seealso \code{\link{calculateOL}}
 
 calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE, parent = NULL) {
+
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Analyse off-ladder alleles"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmKits <- "Select kits"
+  strFrmOptions <- "Options"
+  strLblDatabase <- "Select allele frequency database:"
+  strChkVirtual <- "Include virtual bins in analysis"
+  strTipVirtual <- "NB! Not all vendors specify which alleles are virtual in the bins file. This can be done manually in the kit.txt file."
+  strChkLowFreq <- "Limit small frequencies to 5/2N"
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnCalculate <- "Calculate"
+  strBtnProcessing <- "Processing..."
+  strMsgKit <- "At least one kit must be selected."
+  strMsgTitleKit <- "No kit selected"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmKits"]$value
+    strFrmKits <- ifelse(is.na(strtmp), strFrmKits, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strLblDatabase"]$value
+    strLblDatabase <- ifelse(is.na(strtmp), strLblDatabase, strtmp)
+
+    strtmp <- dtStrings["strChkVirtual"]$value
+    strChkVirtual <- ifelse(is.na(strtmp), strChkVirtual, strtmp)
+
+    strtmp <- dtStrings["strTipVirtual"]$value
+    strTipVirtual <- ifelse(is.na(strtmp), strTipVirtual, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnCalculate"]$value
+    strBtnCalculate <- ifelse(is.na(strtmp), strBtnCalculate, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strMsgKit"]$value
+    strMsgKit <- ifelse(is.na(strtmp), strMsgKit, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleKit"]$value
+    strMsgTitleKit <- ifelse(is.na(strtmp), strMsgTitleKit, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Analyse off-ladder alleles", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -89,25 +166,27 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("calculateOL_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Select kits",
+    text = strFrmKits,
     horizontal = TRUE,
     spacing = 5,
-    container = gv
+    container = gv,
+    expand = TRUE,
+    fill = TRUE
   )
 
   scroll_view <- ggroup(
@@ -140,7 +219,7 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
       enabled(analyse_btn) <- TRUE
 
       # Suggest a save name.
-      svalue(f5_save_edt) <- paste(paste(val_kits, collapse = "_"),
+      svalue(save_edt) <- paste(paste(val_kits, collapse = "_"),
         "_OL",
         sep = ""
       )
@@ -150,21 +229,21 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
       enabled(analyse_btn) <- FALSE
 
       # Empty save name.
-      svalue(f5_save_edt) <- ""
+      svalue(save_edt) <- ""
     }
   })
 
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 5,
     container = gv
   )
 
   glabel(
-    text = "Select allele frequency database:",
+    text = strLblDatabase,
     anchor = c(-1, 0), container = f1
   )
 
@@ -176,45 +255,33 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
   )
 
   f1_virtual_chk <- gcheckbox(
-    text = "Include virtual bins in analysis",
+    text = strChkVirtual,
     checked = TRUE,
     container = f1
   )
-
-  f1_msg <- paste(
-    "NB! Not all vendors specify which alleles are virtual",
-    "in the bins file.\n",
-    "This can be done manually in the kit.txt file."
-  )
-  glabel(text = f1_msg, anchor = c(-1, 0), container = f1)
+  tooltip(f1_virtual_chk) <- strTipVirtual
 
   f1_limit_chk <- gcheckbox(
-    text = "Limit small frequencies to 5/2N",
+    text = strChkLowFreq,
     checked = TRUE,
     container = f1
   )
 
-  # FRAME 5 ###################################################################
+  # SAVE ######################################################################
 
-  f5 <- gframe(
-    text = "Save as",
-    horizontal = TRUE,
-    spacing = 5,
-    container = gv
-  )
+  save_frame <- gframe(text = strFrmSave, container = gv)
 
-  glabel(text = "Name for result:", container = f5)
-
-  f5_save_edt <- gedit(expand = TRUE, container = f5, fill = TRUE)
+  glabel(text = strLblSave, container = save_frame)
 
+  save_edt <- gedit(expand = TRUE, fill = TRUE, container = save_frame)
 
   # BUTTON ####################################################################
 
 
-  analyse_btn <- gbutton(text = "Analyse", container = gv)
+  analyse_btn <- gbutton(text = strBtnCalculate, container = gv)
 
   addHandlerClicked(analyse_btn, handler = function(h, ...) {
-    val_name <- svalue(f5_save_edt)
+    val_name <- svalue(save_edt)
     val_kits <- svalue(kit_checkbox_group)
     val_kitData <- data.frame() # Filled further down.
     val_db_selected <- svalue(f1_db_drp)
@@ -226,7 +293,7 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
 
       # Change button.
       blockHandlers(analyse_btn)
-      svalue(analyse_btn) <- "Processing..."
+      svalue(analyse_btn) <- strBtnProcessing
       unblockHandlers(analyse_btn)
       enabled(analyse_btn) <- FALSE
 
@@ -265,7 +332,7 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
       # Update audit trail.
       datanew <- auditTrail(
         obj = datanew, key = keys, value = values,
-        label = "calculateOL_gui", arguments = FALSE,
+        label = fnc, arguments = FALSE,
         package = "strvalidator"
       )
 
@@ -274,17 +341,16 @@ calculateOL_gui <- function(env = parent.frame(), savegui = NULL, debug = TRUE,
 
       if (debug) {
         print(str(datanew))
-        print(paste("EXIT:", match.call()[[1]]))
+        print(paste("EXIT:", fnc))
       }
 
       # Close GUI.
       .saveSettings()
       dispose(w)
     } else {
-      message <- "At least one kit has to be selected."
-
-      gmessage(message,
-        title = "Not kit selected",
+      gmessage(
+        mmsg = strMsgKit,
+        title = strMsgTitleKit,
         icon = "error",
         parent = w
       )
