@@ -24,13 +24,13 @@
 #' multiple kit names can be given as argument 'kit'. The specified kits will
 #' be used to estimate the size of each allele. If 'kit' is NULL the kit(s)
 #' will be automatically detected, and the 'Size' will be calculated.
-#' 
+#'
 #' The column 'Group' can be used to separate datasets to be compared, and if
 #' so 'kit' must be a vector of equal length as the number of groups, and in
 #' the same order. If not the first 'kit' will be recycled for all groups.
-#' 
-#' Data will be filtered using the reference profiles. 
-#' 
+#'
+#' Data will be filtered using the reference profiles.
+#'
 #' @param data data.frame with at least columns 'Sample.Name', 'Marker', and
 #' 'Height'.
 #' @param ref data.frame with at least columns 'Sample.Name', 'Marker',
@@ -43,18 +43,17 @@
 #' used for all groups.
 #' @param debug logical indicating printing debug information.
 #' @param ... additional arguments to the \code{\link{filterProfile}} function
-#' 
+#'
 #' @return data.frame with with columns 'Sample.Name', 'Kit', 'Group', 'Slope',
 #' 'Error', 'Peaks', 'Lower', and 'Upper'.
-#' 
+#'
 #' @importFrom utils str
 #' @importFrom data.table data.table
-#' 
+#'
 #' @export
 
-calculateSlope <-  function(data, ref, conf = 0.975, kit = NULL, debug=FALSE, ...){
-
-  if(debug){
+calculateSlope <- function(data, ref, conf = 0.975, kit = NULL, debug = FALSE, ...) {
+  if (debug) {
     print(paste("IN:", match.call()[[1]]))
     print("data")
     print(str(data))
@@ -65,205 +64,205 @@ calculateSlope <-  function(data, ref, conf = 0.975, kit = NULL, debug=FALSE, ..
     print("kit")
     print(kit)
   }
-  
+
   # CHECK DATA ----------------------------------------------------------------
-  
+
   # Check dataset.
-  if(!any(grepl("Sample.Name", names(data)))){
+  if (!any(grepl("Sample.Name", names(data)))) {
     stop("'data' must contain a column 'Sample.Name'.",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
 
-  if(!any(grepl("Allele", names(data)))){
+  if (!any(grepl("Allele", names(data)))) {
     stop("'data' must contain a column 'Allele'.",
-         call. = TRUE)
-  }
-  
-  # Check if slim format.
-  if(sum(grepl("Allele", names(data))>1)){
-    stop("'data' must be in 'slim' format.",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
 
-  if(!any(grepl("Height", names(data)))){
-    stop("'data' must contain a column 'Height'.",
-         call. = TRUE)
-  }
-  
   # Check if slim format.
-  if(sum(grepl("Height", names(data))>1)){
+  if (sum(grepl("Allele", names(data)) > 1)) {
     stop("'data' must be in 'slim' format.",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
-  
+
+  if (!any(grepl("Height", names(data)))) {
+    stop("'data' must contain a column 'Height'.",
+      call. = TRUE
+    )
+  }
+
+  # Check if slim format.
+  if (sum(grepl("Height", names(data)) > 1)) {
+    stop("'data' must be in 'slim' format.",
+      call. = TRUE
+    )
+  }
+
   # Check if character data.
-  if(!is.numeric(data$Height)){
+  if (!is.numeric(data$Height)) {
     message("'Height' must be numeric. 'data' converted.")
     data$Height <- as.numeric(data$Height)
   }
 
   # Check dataset.
-  if(!any(grepl("Sample.Name", names(ref)))){
+  if (!any(grepl("Sample.Name", names(ref)))) {
     stop("'ref' must contain a column 'Sample.Name'.",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
-  
-  if(!any(grepl("Allele", names(ref)))){
+
+  if (!any(grepl("Allele", names(ref)))) {
     stop("'ref' must contain a column 'Allele'.",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
-  
+
   # Check if slim format.
-  if(sum(grepl("Allele", names(ref))>1)){
+  if (sum(grepl("Allele", names(ref)) > 1)) {
     stop("'ref' must be in 'slim' format.",
-         call. = TRUE)
+      call. = TRUE
+    )
   }
 
   # Check if character data.
-  if(!is.numeric(conf)){
+  if (!is.numeric(conf)) {
     stop("'conf' must be numeric.")
   }
-  
+
   # PREPARE -----------------------------------------------------------------
 
   # Add a column for kit name.
   data$Kit <- NA
-  
-  # Add group if not present.  
-  if(is.null(data$Group)){
-    
+
+  # Add group if not present.
+  if (is.null(data$Group)) {
     data$Group <- "1"
-    
   }
-  
+
   # Get groups.
   group <- unique(data$Group)
 
   # Filter known profile.
-  data <- filterProfile(data = data, ref = ref,
-                        add.missing.loci = FALSE, keep.na = FALSE,
-                        debug = debug, ...)
+  data <- filterProfile(
+    data = data, ref = ref,
+    add.missing.loci = FALSE, keep.na = FALSE,
+    debug = debug, ...
+  )
 
   # Check if a size column exist.
-  if(is.null(data$Size)){
-    
+  if (is.null(data$Size)) {
     message("'Size' not in dataset.")
 
     # Add a size column.
-    #data$Size <- NA
+    # data$Size <- NA
 
     # Check if kit is specified.
-    if(is.null(kit)){
-
+    if (is.null(kit)) {
       message("'kit' not specified.")
-      
+
       kit <- rep(NA, length(group))
 
-      # Loop over groups.        
-      for(g in seq(along=group)){
-        
+      # Loop over groups.
+      for (g in seq(along = group)) {
+
         # Auto detect kit. If multiple matches, use the first.
-        kit[g] <- detectKit(data = data[data$Group==group[g],],
-                            debug = debug)[1]
-        
+        kit[g] <- detectKit(
+          data = data[data$Group == group[g], ],
+          debug = debug
+        )[1]
       }
-        
     } else {
-      
-      # Check number of groups matches number of given kits. 
-      if(length(group) != length(kit)){
-        
+
+      # Check number of groups matches number of given kits.
+      if (length(group) != length(kit)) {
         kit <- rep(kit[1], length(group))
-        
       }
-      
     }
 
     # Create a new dataframe to store result.
-    data.new <- data.frame(data[0,])
+    data.new <- data.frame(data[0, ])
 
     # Loop over groups.
-    for(g in seq(along=group)){
-      
+    for (g in seq(along = group)) {
+
       # Get kit information.
       kitData <- getKit(kit = kit[g])
-      
+
       # Add size in base pair.
-      data.tmp <- addSize(data = data[data$Group==group[g],],
-                          kit = kitData, debug = debug)
-      
+      data.tmp <- addSize(
+        data = data[data$Group == group[g], ],
+        kit = kitData, debug = debug
+      )
+
       # Add kit.
       data.tmp$Kit <- kit[g]
-      
-      if(debug){
+
+      if (debug) {
         print(paste("Kit: ", kit[g]))
         print(head(data.tmp))
         print(tail(data.tmp))
       }
-      
+
       # Combine result.
       data.new <- rbind(data.new, data.tmp)
-      
+
       message("Added size according to ", kit[g], " for group ", group[g], ".")
-      
     }
-    
+
     # Overwrite the old data.frame with the new.
     data <- data.new
-
   }
-  
+
   # Check if NA in Size column.
-  if(any(is.na(data$Size))){
-    
+  if (any(is.na(data$Size))) {
     row1 <- nrow(data)
-    
+
     # Remove rows with no size.
     data <- data[!is.na(data$Size), ]
-    
+
     row2 <- nrow(data)
-    
+
     message("Removed ", row1 - row2, " rows with Size=NA")
-    
   }
-  
+
   # Check if 0 in Height column.
-  if(any(data$Height==0)){
-    
+  if (any(data$Height == 0)) {
     row1 <- nrow(data)
-    
+
     # Remove rows with zero height.
-    data <- data[!data$Height==0, ]
-    
+    data <- data[!data$Height == 0, ]
+
     row2 <- nrow(data)
-    
+
     message("Removed ", row1 - row2, " rows with Height=0")
-    
   }
-  
+
   # ANALYSE -------------------------------------------------------------------
-  
+
   # Convert to data table for performance.
   DT <- data.table(data)
-  
+
   # Calculate slope.
-  DT <- DT[, list(Slope=summary(lm(log(Height) ~ Size))$coefficients[2],
-                  Error=summary(lm(log(Height) ~ Size))$coefficients[4],
-                  Peaks=.N), by=list(Sample.Name, Group, Kit)]
-    
+  DT <- DT[, list(
+    Slope = summary(lm(log(Height) ~ Size))$coefficients[2],
+    Error = summary(lm(log(Height) ~ Size))$coefficients[4],
+    Peaks = .N
+  ), by = list(Sample.Name, Group, Kit)]
+
   # Calculate confidence interval.
-  DT[, Lower:=Slope - Error * qt(conf, Peaks - 2), by=Sample.Name]
-  DT[, Upper:=Slope + Error * qt(conf, Peaks - 2), by=Sample.Name]
-  
+  DT[, Lower := Slope - Error * qt(conf, Peaks - 2), by = Sample.Name]
+  DT[, Upper := Slope + Error * qt(conf, Peaks - 2), by = Sample.Name]
+
   # Convert back to data.frame.
   res <- as.data.frame(DT)
-  
+
   # Add attributes to result.
-  attr(data, which="kit") <- kit
-  
+  attr(data, which = "kit") <- kit
+
   # Update audit trail.
   res <- auditTrail(obj = res, f.call = match.call(), package = "strvalidator")
 
   return(res)
-  
-}
\ No newline at end of file
+}
