@@ -1,5 +1,6 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 07.03.2020: Added language support.
 # 17.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 07.08.2017: Added audit trail.
 # 13.07.2017: Fixed issue with button handlers.
@@ -35,11 +36,148 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   .gDataName <- NULL
   .gRefName <- NULL
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
-  w <- gwindow(title = "Calculate profile slope", visible = FALSE)
+  # Default strings.
+  strWinTitle <- "Calculate profile slope"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset"
+  strLblDataset <- "Sample dataset:"
+  strDrpDataset <- "<Select dataset>"
+  strLblSamples <- "samples"
+  strLblRefDataset <- "Reference dataset:"
+  strLblRef <- "references"
+  strBtnCheck <- "Check subsetting"
+  strFrmOptions <- "Options"
+  strLblGroups <- "Groups:"
+  strLblConf <- "Confidence limit:"
+  strLblKit <- "Kit to calculate size from:"
+  strChkKit <- "Autodetect"
+  strTipKit <- "Must be checked for multiple kits."
+  strTipKitDrp <- "Not needed if 'Size' is provided in the dataset."
+  strLblMatching <- "Reference sample name matching:"
+  strChkIgnore <- "Ignore case"
+  strChkWord <- "Add word boundaries"
+  strChkExact <- "Exact matching"
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnCalculate <- "Calculate"
+  strBtnProcessing <- "Processing..."
+  strMsgDataset <- "A sample dataset and a reference dataset must be selected."
+  strMsgTitleDataset <- "Dataset not selected"
+  strMsgCheck <- "Data frame is NULL!\n\nMake sure to select a sample dataset and a reference dataset."
+  strWinTitleCheck <- "Check subsetting"
+  strMsgTitleError <- "Error"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDataset"]$value
+    strDrpDataset <- ifelse(is.na(strtmp), strDrpDataset, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strLblRefDataset"]$value
+    strLblRefDataset <- ifelse(is.na(strtmp), strLblRefDataset, strtmp)
+
+    strtmp <- dtStrings["strLblRef"]$value
+    strLblRef <- ifelse(is.na(strtmp), strLblRef, strtmp)
+
+    strtmp <- dtStrings["strBtnCheck"]$value
+    strBtnCheck <- ifelse(is.na(strtmp), strBtnCheck, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strLblGroups"]$value
+    strLblGroups <- ifelse(is.na(strtmp), strLblGroups, strtmp)
+
+    strtmp <- dtStrings["strLblConf"]$value
+    strLblConf <- ifelse(is.na(strtmp), strLblConf, strtmp)
+
+    strtmp <- dtStrings["strLblKit"]$value
+    strLblKit <- ifelse(is.na(strtmp), strLblKit, strtmp)
+
+    strtmp <- dtStrings["strChkKit"]$value
+    strChkKit <- ifelse(is.na(strtmp), strChkKit, strtmp)
+
+    strtmp <- dtStrings["strTipKit"]$value
+    strTipKit <- ifelse(is.na(strtmp), strTipKit, strtmp)
+
+    strtmp <- dtStrings["strTipKitDrp"]$value
+    strTipKitDrp <- ifelse(is.na(strtmp), strTipKitDrp, strtmp)
+
+    strtmp <- dtStrings["strLblMatching"]$value
+    strLblMatching <- ifelse(is.na(strtmp), strLblMatching, strtmp)
+
+    strtmp <- dtStrings["strChkIgnore"]$value
+    strChkIgnore <- ifelse(is.na(strtmp), strChkIgnore, strtmp)
+
+    strtmp <- dtStrings["strChkWord"]$value
+    strChkWord <- ifelse(is.na(strtmp), strChkWord, strtmp)
+
+    strtmp <- dtStrings["strChkExact"]$value
+    strChkExact <- ifelse(is.na(strtmp), strChkExact, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnCalculate"]$value
+    strBtnCalculate <- ifelse(is.na(strtmp), strBtnCalculate, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strMsgDataset"]$value
+    strMsgDataset <- ifelse(is.na(strtmp), strMsgDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleDataset"]$value
+    strMsgTitleDataset <- ifelse(is.na(strtmp), strMsgTitleDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgCheck"]$value
+    strMsgCheck <- ifelse(is.na(strtmp), strMsgCheck, strtmp)
+
+    strtmp <- dtStrings["strWinTitleCheck"]$value
+    strWinTitleCheck <- ifelse(is.na(strtmp), strWinTitleCheck, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -82,22 +220,22 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("calculateSlope_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Dataset",
+    text = strFrmDataset,
     horizontal = FALSE,
     spacing = 5,
     container = gv
@@ -107,11 +245,11 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
   # Datasets ------------------------------------------------------------------
 
-  f0g0[1, 1] <- glabel(text = "Select dataset:", container = f0g0)
+  f0g0[1, 1] <- glabel(text = strLblDataset, container = f0g0)
 
   f0g0[1, 2] <- f0_dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -123,7 +261,10 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
     ellipsize = "none"
   )
 
-  f0g0[1, 3] <- f0_samples_lbl <- glabel(text = " 0 samples", container = f0g0)
+  f0g0[1, 3] <- f0_samples_lbl <- glabel(
+    text = paste(" 0", strLblSamples),
+    container = f0g0
+  )
 
   addHandlerChanged(f0_dataset_drp, handler = function(h, ...) {
     val_obj <- svalue(f0_dataset_drp)
@@ -142,8 +283,9 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       .gData <<- get(val_obj, envir = env)
       .gDataName <<- val_obj
       samples <- length(unique(.gData$Sample.Name))
-      svalue(f0_samples_lbl) <- paste("", samples, "samples")
+      svalue(f0_samples_lbl) <- paste("", samples, strLblSamples)
       svalue(f2_save_edt) <- paste(.gDataName, "_slope", sep = "")
+      svalue(f1_groups_lbl) <- paste(strLblGroups, unique(.gData$Group))
 
       # Detect kit.
       kitIndex <- detectKit(.gData, index = TRUE)
@@ -154,16 +296,17 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       # Reset components.
       .gData <<- NULL
       svalue(f0_dataset_drp, index = TRUE) <- 1
-      svalue(f0_samples_lbl) <- " 0 samples"
+      svalue(f0_samples_lbl) <- paste(" 0", strLblSamples)
       svalue(f2_save_edt) <- ""
+      svalue(f1_groups_lbl) <- paste(strLblGroups, "")
     }
   })
 
-  f0g0[2, 1] <- glabel(text = "Select reference dataset:", container = f0g0)
+  f0g0[2, 1] <- glabel(text = strLblRefDataset, container = f0g0)
 
   f0g0[2, 2] <- f0_refset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -175,7 +318,10 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
     ellipsize = "none"
   )
 
-  f0g0[2, 3] <- f0_ref_lbl <- glabel(text = " 0 references", container = f0g0)
+  f0g0[2, 3] <- f0_ref_lbl <- glabel(
+    text = paste(" 0", strLblRef),
+    container = f0g0
+  )
 
   addHandlerChanged(f0_refset_drp, handler = function(h, ...) {
     val_obj <- svalue(f0_refset_drp)
@@ -193,23 +339,19 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       .gRef <<- get(val_obj, envir = env)
       .gRefName <<- val_obj
       refs <- length(unique(.gRef$Sample.Name))
-      svalue(f0_ref_lbl) <- paste("", refs, "references")
+      svalue(f0_ref_lbl) <- paste("", refs, strLblRef)
     } else {
 
       # Reset components.
       .gRef <<- NULL
       svalue(f0_refset_drp, index = TRUE) <- 1
-      svalue(f0_ref_lbl) <- " 0 references"
+      svalue(f0_ref_lbl) <- paste(" 0", strLblRef)
     }
   })
 
   # CHECK ---------------------------------------------------------------------
 
-  if (debug) {
-    print("CHECK")
-  }
-
-  f0g0[3, 2] <- f0_check_btn <- gbutton(text = "Check subsetting", container = f0g0)
+  f0g0[3, 2] <- f0_check_btn <- gbutton(text = strBtnCheck, container = f0g0)
 
   addHandlerChanged(f0_check_btn, handler = function(h, ...) {
 
@@ -222,7 +364,7 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
     if (!is.null(.gData) || !is.null(.gRef)) {
       chksubset_w <- gwindow(
-        title = "Check subsetting",
+        title = strWinTitleCheck,
         visible = FALSE, name = title,
         width = NULL, height = NULL, parent = w,
         handler = NULL, action = NULL
@@ -245,9 +387,8 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       visible(chksubset_w) <- TRUE
     } else {
       gmessage(
-        msg = "Data frame is NULL!\n\n
-               Make sure to select a dataset and a reference set",
-        title = "Error",
+        msg = strMsgCheck,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
@@ -256,16 +397,24 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 5,
+    anchor = c(-1, 0),
     container = gv
   )
 
-  f1g1 <- glayout(container = f1, spacing = 1)
+  #----------------------------------------------------------------------------
+
+  f1_groups_lbl <- glabel(text = strLblGroups, anchor = c(-1, 0), container = f1)
+
+  #----------------------------------------------------------------------------
+
+  f1g1 <- glayout(container = f1, spacing = 1, anchor = c(-1, 0))
 
   f1g1[1, 1] <- glabel(
-    text = "Confidence limit:",
+    text = strLblConf,
+    anchor = c(-1, 0),
     container = f1g1
   )
   f1g1[1, 2] <- f1_conf_spn <- gspinbutton(
@@ -276,41 +425,41 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   #----------------------------------------------------------------------------
 
   f1g1[2, 1] <- glabel(
-    text = "Kit to calculate size from:", anchor = c(-1, 0),
+    text = strLblKit, anchor = c(-1, 0),
     container = f1g1
   )
 
   f1g1[3, 1] <- f1_auto_chk <- gcheckbox(
-    text = "Autodetect", checked = FALSE,
+    text = strChkKit, checked = FALSE,
     container = f1g1
   )
-  tooltip(f1_auto_chk) <- "Must be checked for multiple kits."
+  tooltip(f1_auto_chk) <- strTipKit
 
   f1g1[3, 2] <- f1_kit_drp <- gcombobox(
     items = getKit(), selected = 1,
     editable = FALSE, container = f1g1,
     ellipsize = "none"
   )
-  tooltip(f1_kit_drp) <- "Not needed if 'Size' is provided in the dataset."
+  tooltip(f1_kit_drp) <- strTipKitDrp
 
   #----------------------------------------------------------------------------
   glabel(
-    text = "Reference sample name matching:", anchor = c(-1, 0),
+    text = strLblMatching, anchor = c(-1, 0),
     container = f1
   )
 
   f1_ignore_chk <- gcheckbox(
-    text = "Ignore case", checked = TRUE,
+    text = strChkIgnore, checked = TRUE,
     container = f1
   )
 
   f1_word_chk <- gcheckbox(
-    text = "Add word boundaries", checked = FALSE,
+    text = strChkWord, checked = FALSE,
     container = f1
   )
 
   f1_exact_chk <- gcheckbox(
-    text = "Exact matching", checked = FALSE,
+    text = strChkExact, checked = FALSE,
     container = f1
   )
 
@@ -330,15 +479,15 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
   # FRAME 2 ###################################################################
 
-  f2 <- gframe(text = "Save as", horizontal = TRUE, spacing = 5, container = gv)
+  f2 <- gframe(text = strFrmSave, horizontal = TRUE, spacing = 5, container = gv)
 
-  glabel(text = "Name for result:", container = f2)
+  glabel(text = strLblSave, container = f2)
 
   f2_save_edt <- gedit(text = "", container = f2, expand = TRUE, fill = TRUE)
 
   # BUTTON ####################################################################
 
-  calculate_btn <- gbutton(text = "Calculate", container = gv)
+  calculate_btn <- gbutton(text = strBtnCalculate, container = gv)
 
   addHandlerClicked(calculate_btn, handler = function(h, ...) {
 
@@ -359,7 +508,7 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
       # Change button.
       blockHandlers(calculate_btn)
-      svalue(calculate_btn) <- "Processing..."
+      svalue(calculate_btn) <- strBtnProcessing
       unblockHandlers(calculate_btn)
       enabled(calculate_btn) <- FALSE
 
@@ -389,7 +538,7 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       # Update audit trail.
       datanew <- auditTrail(
         obj = datanew, key = keys, value = values,
-        label = "calculateSlope_gui", arguments = FALSE,
+        label = fnc, arguments = FALSE,
         package = "strvalidator"
       )
 
@@ -398,17 +547,16 @@ calculateSlope_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
       if (debug) {
         print(str(datanew))
-        print(paste("EXIT:", match.call()[[1]]))
+        print(paste("EXIT:", fnc))
       }
 
       # Close GUI.
       .saveSettings()
       dispose(w)
     } else {
-      message <- "A dataset must be selected."
-
-      gmessage(message,
-        title = "Datasets not selected",
+      gmessage(
+        msg = strMsgDataset,
+        title = strMsgTitleDataset,
         icon = "error",
         parent = w
       )
