@@ -1,5 +1,6 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 08.03.2020: Added language support.
 # 02.03.2019: Tweaked widgets under tcltk.
 # 17.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 10.07.2018: Fixed error replacing NA's.
@@ -19,7 +20,6 @@
 # 28.06.2014: Added help button and moved save gui checkbox.
 # 07.05.2014: Implemented 'checkDataset'.
 # 07.05.2014: Fixed 'Target Value' drop not updated.
-# 07.02.2014: Removed redundant handler for 'f1_column_drp'.
 
 #' @title Crop Or Replace
 #'
@@ -55,12 +55,217 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
   .gData <- NULL
   .gDataName <- NULL
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Crop or replace data"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset"
+  strLblDataset <- "Dataset:"
+  strDrpDataset <- "<Select dataset>"
+  strLblSamples <- "samples,"
+  strLblColumns <- "columns,"
+  strLblRows <- "rows"
+  strFrmColumn <- "Column"
+  strLblTargetCol <- "Target column:"
+  strDrpColumn <- "<Select column>"
+  strLblInfo <- "Info:"
+  strLblMin <- "Min:"
+  strLblMax <- "Max:"
+  strChkNa <- "Ignore NA in info"
+  strFrmOptions <- "Options"
+  strChkRemNA <- "Remove NA"
+  strLblAction <- "Action:"
+  strRadDiscard <- "Discard values"
+  strRadReplace <- "Replace values"
+  strDrpAbove <- "above"
+  strDrpAboveEqual <- "above or equal to"
+  strDrpBelow <- "below"
+  strDrpBelowEqual <- "below or equal to"
+  strDrpEqual <- "equal to"
+  strDrpNotEqual <- "not equal to"
+  strDrpNA <- "is NA"
+  strDrpNotNA <- "is not NA"
+  strDrpContain <- "containing"
+  strDrpNotContain <- "not containing"
+  strDrpTarget <- "<Target value>"
+  strLblWith <- "with"
+  strLblTarget <- "Target column contain data of type:"
+  strRadNumeric <- "Numeric"
+  strRadCharacter <- "Character"
+  strBtnApply <- "Apply"
+  strBtnProcessing <- "Processing..."
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnSave <- "Save"
+  strBtnSaved <- "Saved!"
+  strMsgDrop <- "Do not make subsets of a drop-out dataset before modelling!\n1) Make a subset from the original data.\n2) Run calculate dropout on that subset.\n3) Model drop-out from the new drop-out dataset."
+  strMsgTitleDrop <- "Warning!"
+  strMsgDataset <- "A dataset must be selected."
+  strMsgTitleDataset <- "Dataset not selected"
+  strMsgTitleError <- "Error"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDataset"]$value
+    strDrpDataset <- ifelse(is.na(strtmp), strDrpDataset, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strLblColumns"]$value
+    strLblColumns <- ifelse(is.na(strtmp), strLblColumns, strtmp)
+
+    strtmp <- dtStrings["strLblRows"]$value
+    strLblRows <- ifelse(is.na(strtmp), strLblRows, strtmp)
+
+    strtmp <- dtStrings["strFrmColumn"]$value
+    strFrmColumn <- ifelse(is.na(strtmp), strFrmColumn, strtmp)
+
+    strtmp <- dtStrings["strLblTargetCol"]$value
+    strLblTargetCol <- ifelse(is.na(strtmp), strLblTargetCol, strtmp)
+
+    strtmp <- dtStrings["strDrpColumn"]$value
+    strDrpColumn <- ifelse(is.na(strtmp), strDrpColumn, strtmp)
+
+    strtmp <- dtStrings["strLblInfo"]$value
+    strLblInfo <- ifelse(is.na(strtmp), strLblInfo, strtmp)
+
+    strtmp <- dtStrings["strLblMin"]$value
+    strLblMin <- ifelse(is.na(strtmp), strLblMin, strtmp)
+
+    strtmp <- dtStrings["strLblMax"]$value
+    strLblMax <- ifelse(is.na(strtmp), strLblMax, strtmp)
+
+    strtmp <- dtStrings["strChkNa"]$value
+    strChkNa <- ifelse(is.na(strtmp), strChkNa, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strChkRemNA"]$value
+    strChkRemNA <- ifelse(is.na(strtmp), strChkRemNA, strtmp)
+
+    strtmp <- dtStrings["strLblAction"]$value
+    strLblAction <- ifelse(is.na(strtmp), strLblAction, strtmp)
+
+    strtmp <- dtStrings["strRadDiscard"]$value
+    strRadDiscard <- ifelse(is.na(strtmp), strRadDiscard, strtmp)
+
+    strtmp <- dtStrings["strRadReplace"]$value
+    strRadReplace <- ifelse(is.na(strtmp), strRadReplace, strtmp)
+
+    strtmp <- dtStrings["strDrpAbove"]$value
+    strDrpAbove <- ifelse(is.na(strtmp), strDrpAbove, strtmp)
+
+    strtmp <- dtStrings["strDrpAboveEqual"]$value
+    strDrpAboveEqual <- ifelse(is.na(strtmp), strDrpAboveEqual, strtmp)
+
+    strtmp <- dtStrings["strDrpBelow"]$value
+    strDrpBelow <- ifelse(is.na(strtmp), strDrpBelow, strtmp)
+
+    strtmp <- dtStrings["strDrpBelowEqual"]$value
+    strDrpBelowEqual <- ifelse(is.na(strtmp), strDrpBelowEqual, strtmp)
+
+    strtmp <- dtStrings["strDrpEqual"]$value
+    strDrpEqual <- ifelse(is.na(strtmp), strDrpEqual, strtmp)
+
+    strtmp <- dtStrings["strDrpNotEqual"]$value
+    strDrpNotEqual <- ifelse(is.na(strtmp), strDrpNotEqual, strtmp)
+
+    strtmp <- dtStrings["strDrpNA"]$value
+    strDrpNA <- ifelse(is.na(strtmp), strDrpNA, strtmp)
+
+    strtmp <- dtStrings["strDrpNotNA"]$value
+    strDrpNotNA <- ifelse(is.na(strtmp), strDrpNotNA, strtmp)
+
+    strtmp <- dtStrings["strDrpContain"]$value
+    strDrpContain <- ifelse(is.na(strtmp), strDrpContain, strtmp)
+
+    strtmp <- dtStrings["strDrpNotContain"]$value
+    strDrpNotContain <- ifelse(is.na(strtmp), strDrpNotContain, strtmp)
+
+    strtmp <- dtStrings["strDrpTarget"]$value
+    strDrpTarget <- ifelse(is.na(strtmp), strDrpTarget, strtmp)
+
+    strtmp <- dtStrings["strLblWith"]$value
+    strLblWith <- ifelse(is.na(strtmp), strLblWith, strtmp)
+
+    strtmp <- dtStrings["strLblTarget"]$value
+    strLblTarget <- ifelse(is.na(strtmp), strLblTarget, strtmp)
+
+    strtmp <- dtStrings["strRadNumeric"]$value
+    strRadNumeric <- ifelse(is.na(strtmp), strRadNumeric, strtmp)
+
+    strtmp <- dtStrings["strRadCharacter"]$value
+    strRadCharacter <- ifelse(is.na(strtmp), strRadCharacter, strtmp)
+
+    strtmp <- dtStrings["strBtnApply"]$value
+    strBtnApply <- ifelse(is.na(strtmp), strBtnApply, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnSave"]$value
+    strBtnSave <- ifelse(is.na(strtmp), strBtnSave, strtmp)
+
+    strtmp <- dtStrings["strBtnSaved"]$value
+    strBtnSaved <- ifelse(is.na(strtmp), strBtnSaved, strtmp)
+
+    strtmp <- dtStrings["strMsgDrop"]$value
+    strMsgDrop <- ifelse(is.na(strtmp), strMsgDrop, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleDrop"]$value
+    strMsgTitleDrop <- ifelse(is.na(strtmp), strMsgTitleDrop, strtmp)
+
+    strtmp <- dtStrings["strMsgDataset"]$value
+    strMsgDataset <- ifelse(is.na(strtmp), strMsgDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleDataset"]$value
+    strMsgTitleDataset <- ifelse(is.na(strtmp), strMsgTitleDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Crop or replace values in data frames", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -104,22 +309,22 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("cropData_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Datasets",
+    text = strFrmDataset,
     horizontal = FALSE,
     spacing = 2,
     container = gv
@@ -127,11 +332,11 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
   g0 <- glayout(container = f0, spacing = 1)
 
-  g0[1, 1] <- glabel(text = "Select dataset:", container = g0)
+  g0[1, 1] <- glabel(text = strLblDataset, container = g0)
 
   g0[1, 2] <- dataset_drp <- gcombobox(
     items = c(
-      "<Select data frame>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -143,9 +348,13 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
     ellipsize = "none"
   )
 
-  g0[1, 3] <- g0_samples_lbl <- glabel(text = " 0 samples,", container = g0)
-  g0[1, 4] <- g0_columns_lbl <- glabel(text = " 0 columns,", container = g0)
-  g0[1, 5] <- g0_rows_lbl <- glabel(text = " 0 rows", container = g0)
+  g0_samples_lbl <- glabel(text = paste(" <NA>", strLblSamples), container = g0)
+  g0_columns_lbl <- glabel(text = paste(" <NA>", strLblColumns), container = g0)
+  g0_rows_lbl <- glabel(text = paste(" <NA>", strLblRows), container = g0)
+
+  g0[1, 3] <- g0_samples_lbl
+  g0[1, 4] <- g0_columns_lbl
+  g0[1, 5] <- g0_rows_lbl
 
   addHandlerChanged(dataset_drp, handler = function(h, ...) {
     val_obj <- svalue(dataset_drp)
@@ -164,14 +373,9 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
       # Check for dropout dataset and warn.
       if ("Dropout" %in% names(.gData)) {
-        message <- paste("Do not make subsets of a drop-out dataset before modelling!",
-          "1) Make a subset from the original data.",
-          "2) Run calculate dropout on that subset.",
-          "3) Model drop-out from the new drop-out dataset.",
-          sep = "\n  "
-        )
-        gmessage(message,
-          title = "Warning!",
+        gmessage(
+          msg = strMsgDrop,
+          title = strMsgTitleDrop,
           icon = "warning", parent = w
         )
       }
@@ -181,27 +385,27 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
       # Update info prior to action.
       samples <- length(unique(.gData$Sample.Name))
-      svalue(g0_samples_lbl) <- paste(" ", samples, "samples,")
-      svalue(g0_columns_lbl) <- paste(" ", ncol(.gData), "columns,")
-      svalue(g0_rows_lbl) <- paste(" ", nrow(.gData), "rows")
+      svalue(g0_samples_lbl) <- paste(" ", samples, strLblSamples)
+      svalue(g0_columns_lbl) <- paste(" ", ncol(.gData), strLblColumns)
+      svalue(g0_rows_lbl) <- paste(" ", nrow(.gData), strLblRows)
       .refresh_column_drp()
 
       # Update 'Save as'.
       svalue(save_edt) <- .gDataName
       samples <- length(unique(.gData$Sample.Name))
-      svalue(f3_samples_lbl) <- paste(" ", samples, "samples,")
-      svalue(f3_columns_lbl) <- paste(" ", ncol(.gData), "columns,")
-      svalue(f3_rows_lbl) <- paste(" ", nrow(.gData), "rows")
+      svalue(f3_samples_lbl) <- paste(" ", samples, strLblSamples)
+      svalue(f3_columns_lbl) <- paste(" ", ncol(.gData), strLblColumns)
+      svalue(f3_rows_lbl) <- paste(" ", nrow(.gData), strLblRows)
 
       # Update info:
-      svalue(f1_min_lbl) <- " Min:"
-      svalue(f1_max_lbl) <- " Max:"
+      svalue(f1_min_lbl) <- strLblMin
+      svalue(f1_max_lbl) <- strLblMax
     } else {
       .gData <<- NULL
       .gDataName <<- NULL
-      svalue(g0_samples_lbl) <- paste(" ", "<NA>", "samples,")
-      svalue(g0_columns_lbl) <- paste(" ", "<NA>", "columns,")
-      svalue(g0_rows_lbl) <- paste(" ", "<NA>", "rows")
+      svalue(g0_samples_lbl) <- paste(" <NA>", strLblSamples)
+      svalue(g0_columns_lbl) <- paste(" <NA>", strLblColumns)
+      svalue(g0_rows_lbl) <- paste(" <NA>", strLblRows)
 
       # Update info:
       svalue(f1_min_lbl) <- " Min:"
@@ -209,35 +413,35 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
       # Update 'Save as'.
       svalue(save_edt) <- ""
-      svalue(f3_samples_lbl) <- paste(" ", "<NA>", "samples,")
-      svalue(f3_columns_lbl) <- paste(" ", "<NA>", "columns,")
-      svalue(f3_rows_lbl) <- paste(" ", "<NA>", "rows")
+      svalue(f3_samples_lbl) <- paste(" <NA>", strLblSamples)
+      svalue(f3_columns_lbl) <- paste(" <NA>", strLblColumns)
+      svalue(f3_rows_lbl) <- paste(" <NA>", strLblRows)
     }
   })
 
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Column",
+    text = strFrmColumn,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Select target column:", container = f1)
+  glabel(text = strLblTargetCol, container = f1)
 
   f1_column_drp <- gcombobox(
-    items = "<Select column>",
+    items = strDrpColumn,
     selected = 1,
     editable = FALSE,
     container = f1,
     ellipsize = "none"
   )
 
-  glabel(text = " Info:", container = f1)
-  f1_min_lbl <- glabel(text = " Min:", container = f1)
-  f1_max_lbl <- glabel(text = " Max:", container = f1)
-  f1_na_chk <- gcheckbox(text = "Ignore NA for info", checked = TRUE, container = f1)
+  glabel(text = strLblInfo, container = f1)
+  f1_min_lbl <- glabel(text = strLblMin, container = f1)
+  f1_max_lbl <- glabel(text = strLblMax, container = f1)
+  f1_na_chk <- gcheckbox(text = strChkNa, checked = TRUE, container = f1)
 
   addHandlerChanged(f1_na_chk, handler = function(h, ...) {
     .refresh_info()
@@ -270,30 +474,30 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
   # FRAME 2 ###################################################################
 
   f2 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 2,
     container = gv
   )
 
-  f2_na_rm_chk <- gcheckbox(text = "Remove NA", checked = FALSE, container = f2)
+  f2_na_rm_chk <- gcheckbox(text = strChkRemNA, checked = FALSE, container = f2)
 
-  glabel(text = "Action:", visible = FALSE, anchor = c(-1, -1), container = f2)
+  glabel(text = strLblAction, visible = FALSE, anchor = c(-1, -1), container = f2)
 
   f2g1 <- glayout(container = f2, spacing = 2)
 
   f2g1[1, 1] <- f2g1_task_opt <- gradio(
-    items = c("Discard values", "Replace values"),
+    items = c(strRadDiscard, strRadReplace),
     selected = 1,
     container = f2g1
   )
 
   f2_items <- c(
-    "above", "above or equal to",
-    "below", "below or equal to",
-    "equal to", "not equal to",
-    "is NA", "is not NA",
-    "containing", "not containing"
+    strDrpAbove, strDrpAboveEqual,
+    strDrpBelow, strDrpBelowEqual,
+    strDrpEqual, strDrpNotEqual,
+    strDrpNA, strDrpNotNA,
+    strDrpContain, strDrpNotContain
   )
 
   f2g1[1, 2] <- f2g1_operator_drp <- gcombobox(
@@ -301,10 +505,8 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
     ellipsize = "none"
   )
 
-  # Note: <Target> is only to create width of the widget.
-  #       It will be replaced when a target column is selected.
   f2g1[1, 3] <- f2g1_target_cbo <- gcombobox(
-    items = "<Target value>",
+    items = strDrpTarget,
     selected = 1,
     editable = TRUE,
     container = f2g1,
@@ -312,7 +514,7 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
   )
 
   f2g1[1, 4] <- f2g1_new_lbl <- glabel(
-    text = "with",
+    text = strLblWith,
     visible = FALSE,
     anchor = c(0, -1),
     container = f2g1
@@ -326,12 +528,12 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
   )
 
   glabel(
-    text = "Target column contain data of type:",
+    text = strLblTarget,
     visible = FALSE, anchor = c(-1, -1), container = f2
   )
 
   f2_type_opt <- gradio(
-    items = c("Numeric", "Character"),
+    items = c(strRadNumeric, strRadCharacter),
     horizontal = FALSE,
     selected = 1,
     container = f2
@@ -349,7 +551,7 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
   # BUTTON ####################################################################
 
-  apply_btn <- gbutton(text = "Apply", container = gv)
+  apply_btn <- gbutton(text = strBtnApply, container = gv)
 
 
   addHandlerClicked(apply_btn, handler = function(h, ...) {
@@ -402,12 +604,12 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
     # Change button.
     blockHandlers(apply_btn)
-    svalue(apply_btn) <- "Processing..."
+    svalue(apply_btn) <- strBtnProcessing
     unblockHandlers(apply_btn)
     enabled(apply_btn) <- FALSE
 
     blockHandlers(save_btn)
-    svalue(save_btn) <- "Save"
+    svalue(save_btn) <- strBtnSave
     unblockHandlers(save_btn)
 
     if (!is.null(.gData) && !is.null(.gData)) {
@@ -514,7 +716,7 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
       # Update audit trail.
       .gData <- auditTrail(
         obj = .gData, key = keys, value = values,
-        label = "cropData_gui", arguments = FALSE,
+        label = fnc, arguments = FALSE,
         package = "strvalidator"
       )
 
@@ -530,15 +732,15 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
       }
     } else {
       gmessage(
-        msg = "Data frame is NULL or NA!",
-        title = "Error",
+        msg = strMsgDataset,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
 
     # Change button.
     blockHandlers(apply_btn)
-    svalue(apply_btn) <- "Apply"
+    svalue(apply_btn) <- strBtnApply
     unblockHandlers(apply_btn)
     enabled(apply_btn) <- TRUE
 
@@ -554,17 +756,17 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
 
   # SAVE ######################################################################
 
-  save_frame <- gframe(text = "Save as", container = gv)
+  save_frame <- gframe(text = strFrmSave, container = gv)
 
-  glabel(text = "Name for result:", container = save_frame)
+  glabel(text = strLblSave, container = save_frame)
 
   save_edt <- gedit(expand = TRUE, fill = TRUE, container = save_frame)
 
-  save_btn <- gbutton(text = "Save", container = save_frame)
+  save_btn <- gbutton(text = strBtnSave, container = save_frame)
 
-  f3_samples_lbl <- glabel(text = " 0 samples,", container = save_frame)
-  f3_columns_lbl <- glabel(text = " 0 columns,", container = save_frame)
-  f3_rows_lbl <- glabel(text = " 0 rows", container = save_frame)
+  f3_samples_lbl <- glabel(text = paste(" <NA>", strLblSamples), container = save_frame)
+  f3_columns_lbl <- glabel(text = paste(" <NA>", strLblColumns), container = save_frame)
+  f3_rows_lbl <- glabel(text = paste(" <NA>", strLblRows), container = save_frame)
 
   addHandlerChanged(save_btn, handler = function(h, ...) {
     val_name <- svalue(save_edt)
@@ -583,7 +785,7 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
     # Save data.
     saveObject(name = val_name, object = datanew, parent = w, env = env)
     blockHandlers(save_btn)
-    svalue(save_btn) <- "Saved!"
+    svalue(save_btn) <- strBtnSaved
     unblockHandlers(save_btn)
   })
 
@@ -601,7 +803,7 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
       blockHandler(f1_column_drp)
 
       # Populate drop list.
-      f1_column_drp[] <- c("<Select column>", dfs)
+      f1_column_drp[] <- c(strDrpColumn, dfs)
 
       # Select default value.
       svalue(f1_column_drp, index = TRUE) <- 1
@@ -628,42 +830,42 @@ cropData_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, pa
         # Update info:
         if (is.factor(.gData[, val_col])) {
           svalue(f1_min_lbl) <- paste(
-            " Min:",
+            strLblMin,
             min(as.character(.gData[, val_col]),
               na.rm = val_na
             )
           )
           svalue(f1_max_lbl) <- paste(
-            " Max:",
+            strLblMax,
             max(as.character(.gData[, val_col]),
               na.rm = val_na
             )
           )
         } else {
           svalue(f1_min_lbl) <- paste(
-            " Min:",
+            strLblMin,
             min(.gData[, val_col], na.rm = val_na)
           )
           svalue(f1_max_lbl) <- paste(
-            " Max:",
+            strLblMax,
             max(.gData[, val_col], na.rm = val_na)
           )
         }
 
         # Update 'Save As'
         samples <- length(unique(.gData$Sample.Name))
-        svalue(f3_samples_lbl) <- paste(" ", samples, "samples,")
-        svalue(f3_columns_lbl) <- paste(" ", ncol(.gData), "columns,")
-        svalue(f3_rows_lbl) <- paste(" ", nrow(.gData), "rows")
+        svalue(f3_samples_lbl) <- paste(" ", samples, strLblSamples)
+        svalue(f3_columns_lbl) <- paste(" ", ncol(.gData), strLblColumns)
+        svalue(f3_rows_lbl) <- paste(" ", nrow(.gData), strLblRows)
       } else {
 
         # Update info:
-        svalue(f1_min_lbl) <- " Min:"
-        svalue(f1_max_lbl) <- " Max:"
+        svalue(f1_min_lbl) <- strLblMin
+        svalue(f1_max_lbl) <- strLblMax
 
-        svalue(f3_samples_lbl) <- paste(" ", "<NA>", "samples,")
-        svalue(f3_columns_lbl) <- paste(" ", "<NA>", "columns,")
-        svalue(f3_rows_lbl) <- paste(" ", "<NA>", "rows")
+        svalue(f3_samples_lbl) <- paste(" <NA>", strLblSamples)
+        svalue(f3_columns_lbl) <- paste(" <NA>", strLblColumns)
+        svalue(f3_rows_lbl) <- paste(" <NA>", strLblRows)
       }
     }
 
