@@ -39,15 +39,15 @@
 #' can be imported.
 #' NB! Empty strings ("") and NA strings ("NA") are converted to NA.
 #' See \code{\link{list.files}} and \code{\link{read.table}} for additional details.
-#' 
+#'
 #' @param folder logical, TRUE all files in folder will be imported,
 #' FALSE only selected file will be imported.
 #' @param suffix string, only files with specified suffix will be imported.
 #' @param prefix string, only files with specified prefix will be imported.
 #' @param import.file string if file name is provided file will be imported
-#' without showing the file open dialogue. 
+#' without showing the file open dialogue.
 #' @param folder.name string if folder name is provided files in folder
-#' will be imported without showing the select folder dialogue. 
+#' will be imported without showing the select folder dialogue.
 #' @param extension string providing the file extension.
 #' @param file.name logical if TRUE the file name is written in a column 'File.Name'.
 #' NB! Any existing 'File.Name' column is overwritten.
@@ -65,36 +65,35 @@
 #' @param slim.na logical indicating if rows without data should remain.
 #' @param na.strings character vector with strings to be replaced by NA.
 #' @param debug logical indicating printing debug information.
-#' 
-#' 
+#'
+#'
 #' @return data.frame with imported result.
-#' 
+#'
 #' @export
-#' 
+#'
 #' @importFrom plyr rbind.fill
 #' @importFrom utils read.table
 # @importFrom data.table fread
-#' 
+#'
 #' @seealso \code{\link{trim}}, \code{\link{slim}}, \code{\link{list.files}}, \code{\link{read.table}}
 
 
 
-import <- function (folder = TRUE, extension="txt", 
-                    suffix = NA, prefix = NA, 
-                    import.file = NA, folder.name = NA,
-                    file.name = TRUE, time.stamp = TRUE,
-                    separator = "\t", ignore.case = TRUE,
-                    auto.trim = FALSE, trim.samples = NULL,
-                    trim.invert = FALSE,
-                    auto.slim = FALSE, slim.na = TRUE,
-                    na.strings = c("NA",""),
-                    debug = FALSE){
-  
-  if(debug){
+import <- function(folder = TRUE, extension = "txt",
+                   suffix = NA, prefix = NA,
+                   import.file = NA, folder.name = NA,
+                   file.name = TRUE, time.stamp = TRUE,
+                   separator = "\t", ignore.case = TRUE,
+                   auto.trim = FALSE, trim.samples = NULL,
+                   trim.invert = FALSE,
+                   auto.slim = FALSE, slim.na = TRUE,
+                   na.strings = c("NA", ""),
+                   debug = FALSE) {
+  if (debug) {
     print(paste("IN:", match.call()[[1]]))
   }
-  
-  if(debug){
+
+  if (debug) {
     print("folder")
     print(folder)
     print("extension")
@@ -114,165 +113,170 @@ import <- function (folder = TRUE, extension="txt",
     print("ignore.case")
     print(ignore.case)
   }
-  
+
   # Check data ----------------------------------------------------------------
-  
+
   # Check data.
-  if(is.na(import.file) && is.na(folder.name)){
+  if (is.na(import.file) && is.na(folder.name)) {
     stop("Either 'import.file' or 'folder.name' must be provided")
   }
-  
+
   # Import --------------------------------------------------------------------
-  
+
   # Initialise result data.frame (no match return empty dataframe)
   res <- data.frame()
-  
+
   # Check if result files in folder.
   if (folder) {
 
-    # Get path.    
+    # Get path.
     folder <- folder.name
 
     # Check if folder is specified.
     if (!is.na(folder)) {
-      
+
       # Create file filter.
-      fileFilter <- paste(".*", sep="")
+      fileFilter <- paste(".*", sep = "")
       if (!is.na(prefix) && nchar(prefix) > 0) {
-        fileFilter <- paste("^", prefix, fileFilter, sep="") 
-        if(debug){
+        fileFilter <- paste("^", prefix, fileFilter, sep = "")
+        if (debug) {
           print("prefix added:")
           print(fileFilter)
-        }  
+        }
       }
       if (!is.na(suffix) && nchar(suffix) > 0) {
-        fileFilter <- paste(fileFilter, suffix, sep="") 
-        if(debug){
+        fileFilter <- paste(fileFilter, suffix, sep = "")
+        if (debug) {
           print("suffix added:")
           print(fileFilter)
-        }  
+        }
       }
-      fileFilter <- paste(fileFilter,"\\.", extension, sep="") 
-      if(debug){
+      fileFilter <- paste(fileFilter, "\\.", extension, sep = "")
+      if (debug) {
         print("fileFilter")
         print(fileFilter)
         print("folder")
         print(folder)
-      }  
-      
+      }
+
       # Get list of files.
-      import.file <- list.files(path = folder, pattern = fileFilter,
-                                full.names = TRUE, recursive = FALSE,
-                                ignore.case = ignore.case, include.dirs = FALSE)
+      import.file <- list.files(
+        path = folder, pattern = fileFilter,
+        full.names = TRUE, recursive = FALSE,
+        ignore.case = ignore.case, include.dirs = FALSE
+      )
     }
-  
   }
-    
-  if(debug){
+
+  if (debug) {
     print("import.file")
     print(import.file)
-  }  
-  
+  }
+
   # Check if files are specified.
   if (any(length(import.file) > 0, !is.na(import.file))) {
 
     # Autotrim message (function inside loop).
-    if(auto.trim){
-      message(paste("Auto trim samples:", trim.samples,
-                    " invert =", trim.invert))
+    if (auto.trim) {
+      message(paste(
+        "Auto trim samples:", trim.samples,
+        " invert =", trim.invert
+      ))
     }
-    
+
     # Read files.
-    for (f in seq(along=import.file)) {
+    for (f in seq(along = import.file)) {
 
       # Should change to more efficient and simpler 'fread' but
       # problem is that autodetection of colClasses does not always work
       # and it is not possible(?) to set all to character.
       # Use read.table for the time being.
-      # Read a file.  
+      # Read a file.
       # tmpdf <- data.table::fread(import.file[f], data.table=FALSE)
-      
+
       # Ensures column names are identical as when read.table was used.
       # Needed since many functions specify columns by name.
       # names(tmpdf) <- make.names(colnames(tmpdf))
 
-      # Read a file.  
-      tmpdf <- read.table(import.file[f], header = TRUE,
-                          sep = separator, fill = TRUE,
-                          na.strings = na.strings,
-                          colClasses = "character",
-                          stringsAsFactors=FALSE)
-      
+      # Read a file.
+      tmpdf <- read.table(import.file[f],
+        header = TRUE,
+        sep = separator, fill = TRUE,
+        na.strings = na.strings,
+        colClasses = "character",
+        stringsAsFactors = FALSE
+      )
+
       # Autotrim datset (message before loop).
-      if(auto.trim){
-        tmpdf <- trim(data=tmpdf, samples=trim.samples,
-                    invert.s=trim.invert, debug=debug)
+      if (auto.trim) {
+        tmpdf <- trim(
+          data = tmpdf, samples = trim.samples,
+          invert.s = trim.invert, debug = debug
+        )
       }
-      
+
       # Show progress.
-      message(paste("Importing (", f, " of ", length(import.file),"): ",
-                    import.file[f], sep=""))
+      message(paste("Importing (", f, " of ", length(import.file), "): ",
+        import.file[f],
+        sep = ""
+      ))
 
       # Check if file path should be saved.
-      if(file.name){
+      if (file.name) {
 
         # Add column and save file name.
         tmpdf$File.Name <- basename(import.file[f])
-        
       }
-      
+
       # Check if time stamp should be saved.
-      if(time.stamp){
-        
+      if (time.stamp) {
+
         # Add column and save file name.
         tmptime <- file.info(import.file[f])
         tmpdf$File.Time <- as.character(tmptime$mtime)
-        
       }
-      
+
       # Check if multiple files.
-      if(f > 1){
+      if (f > 1) {
         # Add to result data frame.
         res <- plyr::rbind.fill(res, tmpdf)
       } else {
         # Create result data frame.
         res <- tmpdf
       }
-      
     }
 
     # Autoslim dataset.
-    if(auto.slim){
-      
+    if (auto.slim) {
+
       # Autodetect column names to keep fixed.
-      fixCol <- colNames(data=res, slim=TRUE, numbered=TRUE, concatenate="|")
-      
+      fixCol <- colNames(data = res, slim = TRUE, numbered = TRUE, concatenate = "|")
+
       # Autodetect column names to stack.
-      stackCol <- colNames(data=res, slim=FALSE, numbered=TRUE, concatenate="|")
-      
+      stackCol <- colNames(data = res, slim = FALSE, numbered = TRUE, concatenate = "|")
+
       # Progress.
       message("Auto slim dataset...")
       message(paste("  Stack columns:", stackCol))
       message(paste("  Fix columns:", fixCol))
-      
+
       # Slim require a vector of strings.
       fixCol <- unlist(strsplit(fixCol, "|", fixed = TRUE))
       stackCol <- unlist(strsplit(stackCol, "|", fixed = TRUE))
-      
-      # Slim data.      
-      res <- slim(data=res, fix=fixCol, stack=stackCol,
-                  keep.na=slim.na, debug=debug)
-      
+
+      # Slim data.
+      res <- slim(
+        data = res, fix = fixCol, stack = stackCol,
+        keep.na = slim.na, debug = debug
+      )
     }
-    
   }
 
   # Update audit trail.
   res <- auditTrail(obj = res, f.call = match.call(), package = "strvalidator")
 
   # Convert common known numeric columns.
-  res <- colConvert(data=res)
-  
+  res <- colConvert(data = res)
+
   return(res)
-  
 }
