@@ -1,5 +1,6 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 10.04.2020: Added language support.
 # 20.03.2019: Fixed save object triggered when plotting if label is changed.
 # 24.02.2019: Added option to use log10 y-axis scale.
 # 23.02.2019: Compacted and tweaked gui for tcltk.
@@ -19,8 +20,6 @@
 # 19.08.2015: Added more information to the description.
 # 18.06.2015: Rounded printed probabilities to three decimals.
 # 20.03.2015: Rounded printed conservative drop-out threshold to integer.
-# 05.01.2015: Changed check of suggested package ResourceSelection in accordance
-#             with Writing R extensions v 3.2.1 section 1.1.3.1.
 
 #' @title Model And Plot Drop-out Events
 #'
@@ -142,12 +141,329 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   .gData <- NULL
   .gPlot <- NULL
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Plot dropout prediction"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset"
+  strLblDataset <- "Dropout dataset:"
+  strDrpDataset <- "<Select dataset>"
+  strLblKit <- "and the kit used:"
+  strFrmOptions <- "Options"
+  strChkOverride <- "Override automatic titles"
+  strLblTitlePlot <- "Plot title:"
+  strLblTitleX <- "X title:"
+  strLblTitleY <- "Y title:"
+  strLblRange <- "Dataset peak height range:"
+  strChkLog <- "Log (Height)"
+  strChkSex <- "Exclude sex markers"
+  strLblNote <- "NB! Currently, the recommended methods are the first three options.\nThe fourth alternative has not been evaluated by the DNA Commission.\nSee 'Details' in 'Help' for more information."
+  strLblModels <- "Model drop-out from scoring method:"
+  strRadRandom <- "Relative a random allele and peak height of surviving allele"
+  strRadLMW <- "Relative the low molecular weight allele and peak height of surviving allele"
+  strRadHMW <- "Relative the high molecular weight allele and peak height of surviving allele"
+  strRadLocus <- "Relative the locus and peak height of surviving allele, or mean locus peak height"
+  strChkAverage <- "Use average peak height 'H' instead of allele/locus peak hight"
+  strChkPrint <- "Print model"
+  strChkDump <- "Dump model input"
+  strExpThreshold <- "Drop-out prediction and threshold"
+  strChkThreshold <- "Mark threshold @ P(D):"
+  strLblLineType <- "Line type"
+  strLblBlank <- "blank"
+  strLblSolid <- "solid"
+  strLblDashed <- "dashed"
+  strLblDotted <- "dotted"
+  strLblDotDash <- "dotdash"
+  strLblLongDash <- "longdash"
+  strLblTwoDash <- "twodash"
+  strLblLineColour <- "Line colour"
+  strChkPrintT <- "Print threshold value"
+  strLblPredictionInterval <- "Prediction interval:"
+  strChkPrintTcons <- "Print conservative T value"
+  strChkPredictionInterval <- "Draw prediction interval:"
+  strLblAlpha <- "Alpha:"
+  strLblFill <- "Fill colour:"
+  strExpPoints <- "Data points"
+  strChkPoints <- "Plot data points"
+  strLblShape <- "Shape:"
+  strLblJitter <- "Jitter (h/v):"
+  strExpAxes <- "Axes"
+  strChkScaleLog <- "Use log10 scale at Y axis"
+  strLblNbMinMax <- "NB! Must provide both min and max value."
+  strLblLimitY <- "Limit Y axis (min-max)"
+  strLblLimitX <- "Limit X axis (min-max)"
+  strExpLabels <- "X labels"
+  strLblSize <- "Text size (pts):"
+  strLblAngle <- "Angle:"
+  strLblJustification <- "Justification (v/h):"
+  strBtnPlot <- "Plot predicted drop-out probability"
+  strBtnProcessing <- "Processing..."
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnSaveObject <- "Save as object"
+  strBtnSaveImage <- "Save as image"
+  strBtnObjectSaved <- "Object saved"
+  strLblMainTitleAverage <- "Drop-out probability as a function of average peak height"
+  strLblXTitleAverage <- "Average peak height 'H', (RFU)"
+  strLblMainTitleHeight <- "Drop-out probability as a function of present-allele height"
+  strLblXTitleHeight <- "Peak height, (RFU)"
+  strLblYTitle <- "Drop-out probability, P(D)"
+  strLblLegend <- "Model parameters:"
+  strLblHosmer <- "\nHosmer-Lemeshow test: p = "
+  strMsgDataset <- "A dataset must be selected."
+  strMsgTitleDataset <- "Dataset not selected"
+  strMsgNull <- "Data frame is NULL or NA!"
+  strMsgTitleError <- "Error"
+  strMsgIncomplete1 <- "Dataset is ok for drop-out analysis.\nHowever, additional columns are required for this analysis:\n"
+  strMsgIncomplete2 <- "\n\nPlease try modeling using another scoring method."
+  strMsgTitleIncomplete <- "Incomplete dataset"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDataset"]$value
+    strDrpDataset <- ifelse(is.na(strtmp), strDrpDataset, strtmp)
+
+    strtmp <- dtStrings["strLblKit"]$value
+    strLblKit <- ifelse(is.na(strtmp), strLblKit, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strChkOverride"]$value
+    strChkOverride <- ifelse(is.na(strtmp), strChkOverride, strtmp)
+
+    strtmp <- dtStrings["strLblTitlePlot"]$value
+    strLblTitlePlot <- ifelse(is.na(strtmp), strLblTitlePlot, strtmp)
+
+    strtmp <- dtStrings["strLblTitleX"]$value
+    strLblTitleX <- ifelse(is.na(strtmp), strLblTitleX, strtmp)
+
+    strtmp <- dtStrings["strLblTitleY"]$value
+    strLblTitleY <- ifelse(is.na(strtmp), strLblTitleY, strtmp)
+
+    strtmp <- dtStrings["strLblRange"]$value
+    strLblRange <- ifelse(is.na(strtmp), strLblRange, strtmp)
+
+    strtmp <- dtStrings["strChkLog"]$value
+    strChkLog <- ifelse(is.na(strtmp), strChkLog, strtmp)
+
+    strtmp <- dtStrings["strChkSex"]$value
+    strChkSex <- ifelse(is.na(strtmp), strChkSex, strtmp)
+
+    strtmp <- dtStrings["strLblNote"]$value
+    strLblNote <- ifelse(is.na(strtmp), strLblNote, strtmp)
+
+    strtmp <- dtStrings["strLblModels"]$value
+    strLblModels <- ifelse(is.na(strtmp), strLblModels, strtmp)
+
+    strtmp <- dtStrings["strRadRandom"]$value
+    strRadRandom <- ifelse(is.na(strtmp), strRadRandom, strtmp)
+
+    strtmp <- dtStrings["strRadLMW"]$value
+    strRadLMW <- ifelse(is.na(strtmp), strRadLMW, strtmp)
+
+    strtmp <- dtStrings["strRadHMW"]$value
+    strRadHMW <- ifelse(is.na(strtmp), strRadHMW, strtmp)
+
+    strtmp <- dtStrings["strRadLocus"]$value
+    strRadLocus <- ifelse(is.na(strtmp), strRadLocus, strtmp)
+
+    strtmp <- dtStrings["strChkAverage"]$value
+    strChkAverage <- ifelse(is.na(strtmp), strChkAverage, strtmp)
+
+    strtmp <- dtStrings["strChkPrint"]$value
+    strChkPrint <- ifelse(is.na(strtmp), strChkPrint, strtmp)
+
+    strtmp <- dtStrings["strChkDump"]$value
+    strChkDump <- ifelse(is.na(strtmp), strChkDump, strtmp)
+
+    strtmp <- dtStrings["strExpThreshold"]$value
+    strExpThreshold <- ifelse(is.na(strtmp), strExpThreshold, strtmp)
+
+    strtmp <- dtStrings["strChkThreshold"]$value
+    strChkThreshold <- ifelse(is.na(strtmp), strChkThreshold, strtmp)
+
+    strtmp <- dtStrings["strLblLineType"]$value
+    strLblLineType <- ifelse(is.na(strtmp), strLblLineType, strtmp)
+
+    strtmp <- dtStrings["strLblBlank"]$value
+    strLblBlank <- ifelse(is.na(strtmp), strLblBlank, strtmp)
+
+    strtmp <- dtStrings["strLblSolid"]$value
+    strLblSolid <- ifelse(is.na(strtmp), strLblSolid, strtmp)
+
+    strtmp <- dtStrings["strLblDashed"]$value
+    strLblDashed <- ifelse(is.na(strtmp), strLblDashed, strtmp)
+
+    strtmp <- dtStrings["strLblDotted"]$value
+    strLblDotted <- ifelse(is.na(strtmp), strLblDotted, strtmp)
+
+    strtmp <- dtStrings["strLblDotDash"]$value
+    strLblDotDash <- ifelse(is.na(strtmp), strLblDotDash, strtmp)
+
+    strtmp <- dtStrings["strLblLongDash"]$value
+    strLblLongDash <- ifelse(is.na(strtmp), strLblLongDash, strtmp)
+
+    strtmp <- dtStrings["strLblTwoDash"]$value
+    strLblTwoDash <- ifelse(is.na(strtmp), strLblTwoDash, strtmp)
+
+    strtmp <- dtStrings["strLblLineColour"]$value
+    strLblLineColour <- ifelse(is.na(strtmp), strLblLineColour, strtmp)
+
+    strtmp <- dtStrings["strChkPrintT"]$value
+    strChkPrintT <- ifelse(is.na(strtmp), strChkPrintT, strtmp)
+
+    strtmp <- dtStrings["strLblPredictionInterval"]$value
+    strLblPredictionInterval <- ifelse(is.na(strtmp), strLblPredictionInterval, strtmp)
+
+    strtmp <- dtStrings["strChkPrintTcons"]$value
+    strChkPrintTcons <- ifelse(is.na(strtmp), strChkPrintTcons, strtmp)
+
+    strtmp <- dtStrings["strChkPredictionInterval"]$value
+    strChkPredictionInterval <- ifelse(is.na(strtmp), strChkPredictionInterval, strtmp)
+
+    strtmp <- dtStrings["strLblAlpha"]$value
+    strLblAlpha <- ifelse(is.na(strtmp), strLblAlpha, strtmp)
+
+    strtmp <- dtStrings["strLblFill"]$value
+    strLblFill <- ifelse(is.na(strtmp), strLblFill, strtmp)
+
+    strtmp <- dtStrings["strExpPoints"]$value
+    strExpPoints <- ifelse(is.na(strtmp), strExpPoints, strtmp)
+
+    strtmp <- dtStrings["strChkPoints"]$value
+    strChkPoints <- ifelse(is.na(strtmp), strChkPoints, strtmp)
+
+    strtmp <- dtStrings["strLblShape"]$value
+    strLblShape <- ifelse(is.na(strtmp), strLblShape, strtmp)
+
+    strtmp <- dtStrings["strLblJitter"]$value
+    strLblJitter <- ifelse(is.na(strtmp), strLblJitter, strtmp)
+
+    strtmp <- dtStrings["strExpAxes"]$value
+    strExpAxes <- ifelse(is.na(strtmp), strExpAxes, strtmp)
+
+    strtmp <- dtStrings["strChkScaleLog"]$value
+    strChkScaleLog <- ifelse(is.na(strtmp), strChkScaleLog, strtmp)
+
+    strtmp <- dtStrings["strLblNbMinMax"]$value
+    strLblNbMinMax <- ifelse(is.na(strtmp), strLblNbMinMax, strtmp)
+
+    strtmp <- dtStrings["strLblLimitY"]$value
+    strLblLimitY <- ifelse(is.na(strtmp), strLblLimitY, strtmp)
+
+    strtmp <- dtStrings["strLblLimitX"]$value
+    strLblLimitX <- ifelse(is.na(strtmp), strLblLimitX, strtmp)
+
+    strtmp <- dtStrings["strExpLabels"]$value
+    strExpLabels <- ifelse(is.na(strtmp), strExpLabels, strtmp)
+
+    strtmp <- dtStrings["strLblSize"]$value
+    strLblSize <- ifelse(is.na(strtmp), strLblSize, strtmp)
+
+    strtmp <- dtStrings["strLblAngle"]$value
+    strLblAngle <- ifelse(is.na(strtmp), strLblAngle, strtmp)
+
+    strtmp <- dtStrings["strLblJustification"]$value
+    strLblJustification <- ifelse(is.na(strtmp), strLblJustification, strtmp)
+
+    strtmp <- dtStrings["strBtnPlot"]$value
+    strBtnPlot <- ifelse(is.na(strtmp), strBtnPlot, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnSaveObject"]$value
+    strBtnSaveObject <- ifelse(is.na(strtmp), strBtnSaveObject, strtmp)
+
+    strtmp <- dtStrings["strBtnSaveImage"]$value
+    strBtnSaveImage <- ifelse(is.na(strtmp), strBtnSaveImage, strtmp)
+
+    strtmp <- dtStrings["strBtnObjectSaved"]$value
+    strBtnObjectSaved <- ifelse(is.na(strtmp), strBtnObjectSaved, strtmp)
+
+    strtmp <- dtStrings["strLblMainTitleAverage"]$value
+    strLblMainTitleAverage <- ifelse(is.na(strtmp), strLblMainTitleAverage, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleAverage"]$value
+    strLblXTitleAverage <- ifelse(is.na(strtmp), strLblXTitleAverage, strtmp)
+
+    strtmp <- dtStrings["strLblMainTitleHeight"]$value
+    strLblMainTitleHeight <- ifelse(is.na(strtmp), strLblMainTitleHeight, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleHeight"]$value
+    strLblXTitleHeight <- ifelse(is.na(strtmp), strLblXTitleHeight, strtmp)
+
+    strtmp <- dtStrings["strLblYTitle"]$value
+    strLblYTitle <- ifelse(is.na(strtmp), strLblYTitle, strtmp)
+
+    strtmp <- dtStrings["strLblLegend"]$value
+    strLblLegend <- ifelse(is.na(strtmp), strLblLegend, strtmp)
+
+    strtmp <- dtStrings["strLblHosmer"]$value
+    strLblHosmer <- ifelse(is.na(strtmp), strLblHosmer, strtmp)
+
+    strtmp <- dtStrings["strMsgDataset"]$value
+    strMsgDataset <- ifelse(is.na(strtmp), strMsgDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleDataset"]$value
+    strMsgTitleDataset <- ifelse(is.na(strtmp), strMsgTitleDataset, strtmp)
+
+    strtmp <- dtStrings["strMsgNull"]$value
+    strMsgNull <- ifelse(is.na(strtmp), strMsgNull, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+
+    strtmp <- dtStrings["strMsgIncomplete1"]$value
+    strMsgIncomplete1 <- ifelse(is.na(strtmp), strMsgIncomplete1, strtmp)
+
+    strtmp <- dtStrings["strMsgIncomplete2"]$value
+    strMsgIncomplete2 <- ifelse(is.na(strtmp), strMsgIncomplete2, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleIncomplete"]$value
+    strMsgTitleIncomplete <- ifelse(is.na(strtmp), strMsgTitleIncomplete, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Plot dropout prediction", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -190,32 +506,32 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("modelDropout_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Dataset",
+    text = strFrmDataset,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Select dataset:", container = f0)
+  glabel(text = strLblDataset, container = f0)
 
   dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -227,7 +543,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     ellipsize = "none"
   )
 
-  glabel(text = " and the kit used:", container = f0)
+  glabel(text = strLblKit, container = f0)
 
   kit_drp <- gcombobox(
     items = getKit(),
@@ -286,14 +602,14 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 2,
     container = gv
   )
 
   titles_chk <- gcheckbox(
-    text = "Override automatic titles.",
+    text = strChkOverride,
     checked = FALSE, container = f1
   )
 
@@ -308,18 +624,18 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   )
 
   # Legends
-  glabel(text = "Plot title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitlePlot, container = titles_group, anchor = c(-1, 0))
   title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
-  glabel(text = "X title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitleX, container = titles_group, anchor = c(-1, 0))
   x_title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
-  glabel(text = "Y title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitleY, container = titles_group, anchor = c(-1, 0))
   y_title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
   # Group 2.
   f1g2 <- ggroup(horizontal = TRUE, spacing = 2, container = f1)
-  glabel(text = "Dataset peak height range:", container = f1g2)
+  glabel(text = strLblRange, container = f1g2)
   f1g2_low_lbl <- glabel(text = "", width = 6, container = f1g2)
   glabel(text = "-", container = f1g2)
   f1g2_high_lbl <- glabel(text = "", width = 6, container = f1g2)
@@ -327,50 +643,41 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
 
   # Other options.
-  log_model <- gcheckbox(text = "Log (Height)", checked = FALSE, container = f1)
+  log_model <- gcheckbox(text = strChkLog, checked = FALSE, container = f1)
 
   f1_sex_chk <- gcheckbox(
-    text = "Exclude sex markers",
+    text = strChkSex,
     checked = TRUE,
     container = f1
   )
 
   glabel(
-    text = paste("NB! Currently, the recommended methods are the first three options.\n",
-      "The fourth alternative has not been evaluated by the DNA Commission.",
-      "\nSee 'Details' in 'Help' for more information.",
-      sep = ""
-    ),
+    text = strLblNote,
     anchor = c(-1, 0), container = f1
   )
 
-  glabel(text = "Model drop-out from scoring method:", anchor = c(-1, 0), container = f1)
+  glabel(text = strLblModels, anchor = c(-1, 0), container = f1)
   f1_column_opt <- gradio(
-    items = c(
-      "Relative a random allele and peak height of surviving allele",
-      "Relative the low molecular weight allele and peak height of surviving allele",
-      "Relative the high molecular weight allele and peak height of surviving allele",
-      "Relative the locus and peak height of surviving allele, or mean locus peak height"
-    ),
+    items = c(strRadRandom, strRadLMW, strRadHMW, strRadLocus),
     selected = 2,
     horizontal = FALSE,
     container = f1
   )
 
   f1_h_chk <- gcheckbox(
-    text = "Use average peak height 'H' instead of allele/locus peak hight",
+    text = strChkAverage,
     checked = FALSE,
     container = f1
   )
 
   f1_printmodel_chk <- gcheckbox(
-    text = "Print model",
+    text = strChkPrint,
     checked = FALSE,
     container = f1
   )
 
   f1_dump_chk <- gcheckbox(
-    text = "Dump model input", checked = FALSE,
+    text = strChkDump, checked = FALSE,
     container = f1
   )
 
@@ -385,7 +692,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # BUTTON ####################################################################
 
   plot_drop_btn <- gbutton(
-    text = "Plot predicted drop-out probability",
+    text = strBtnPlot,
     container = gv
   )
 
@@ -395,21 +702,20 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       enabled(plot_drop_btn) <- FALSE
 
       blockHandlers(plot_drop_btn)
-      svalue(plot_drop_btn) <- "Processing..."
+      svalue(plot_drop_btn) <- strBtnProcessing
       unblockHandlers(plot_drop_btn)
 
       .plotDrop()
 
       blockHandlers(plot_drop_btn)
-      svalue(plot_drop_btn) <- "Plot predicted drop-out probability"
+      svalue(plot_drop_btn) <- strBtnPlot
       unblockHandlers(plot_drop_btn)
 
       enabled(plot_drop_btn) <- TRUE
     } else {
-      message <- paste("Select a drop-out dataset")
-
-      gmessage(message,
-        title = "Could not find dataset",
+      gmessage(
+        msg = strMsgDataset,
+        title = strMsgTitleDataset,
         icon = "error",
         parent = w
       )
@@ -420,26 +726,26 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # FRAME 5 ###################################################################
 
   f5 <- gframe(
-    text = "Save as",
+    text = strFrmSave,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Name for result:", container = f5)
+  glabel(text = strLblSave, container = f5)
 
   f5_save_edt <- gedit(text = "", expand = TRUE, fill = TRUE, container = f5)
 
-  f5_save_btn <- gbutton(text = "Save as object", container = f5)
+  f5_save_btn <- gbutton(text = strBtnSaveObject, container = f5)
 
-  f5_ggsave_btn <- gbutton(text = "Save as image", container = f5)
+  f5_ggsave_btn <- gbutton(text = strBtnSaveImage, container = f5)
 
   addHandlerChanged(f5_save_btn, handler = function(h, ...) {
     val_name <- svalue(f5_save_edt)
 
     # Change button.
     blockHandlers(f5_save_btn)
-    svalue(f5_save_btn) <- "Processing..."
+    svalue(f5_save_btn) <- strBtnProcessing
     unblockHandlers(f5_save_btn)
     enabled(f5_save_btn) <- FALSE
 
@@ -451,7 +757,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
     # Change button.
     blockHandlers(f5_save_btn)
-    svalue(f5_save_btn) <- "Object saved"
+    svalue(f5_save_btn) <- strBtnObjectSaved
     unblockHandlers(f5_save_btn)
   })
 
@@ -471,7 +777,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # EXPAND 1 ##################################################################
 
   e1 <- gexpandgroup(
-    text = "Drop-out prediction and threshold",
+    text = strExpThreshold,
     horizontal = FALSE,
     container = f1
   )
@@ -488,7 +794,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   e1f1g2 <- ggroup(horizontal = TRUE, spacing = 2, container = e1f1)
 
   e1f1_threshold_chk <- gcheckbox(
-    text = "Mark threshold @ P(D):",
+    text = strChkThreshold,
     checked = TRUE,
     container = e1f1g2
   )
@@ -502,9 +808,13 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # Group 3.
   e1f1g3 <- ggroup(horizontal = TRUE, spacing = 2, container = e1f1)
 
-  e1_linetypes <- c("blank", "solid", "dashed", "dotted", "dotdash", "longdash", "twodash")
+  e1_linetypes <- c(
+    strLblBlank, strLblSolid, strLblDashed,
+    strLblDotted, strLblDotDash, strLblLongDash,
+    strLblTwoDash
+  )
 
-  glabel("Line type", container = e1f1g3)
+  glabel(strLblLineType, container = e1f1g3)
 
   e1f1_t_linetype_drp <- gcombobox(
     items = e1_linetypes,
@@ -513,7 +823,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     ellipsize = "none"
   )
 
-  glabel("Line colour", container = e1f1g3)
+  glabel(strLblLineColour, container = e1f1g3)
 
   e1f1_t_linecolor_drp <- gcombobox(
     items = palette(),
@@ -526,7 +836,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   e1f1g4 <- ggroup(horizontal = TRUE, spacing = 2, container = e1f1)
 
   e1f1_print_chk <- gcheckbox(
-    text = "Print threshold value",
+    text = strChkPrintT,
     checked = TRUE,
     container = e1f1g4
   )
@@ -540,7 +850,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   e1f2g1 <- ggroup(horizontal = TRUE, spacing = 2, container = e1f2)
 
 
-  glabel(text = "Prediction interval:", container = e1f2g1)
+  glabel(text = strLblPredictionInterval, container = e1f2g1)
 
   e1f2_conf_spn <- gspinbutton(
     from = 0, to = 1, by = 0.001,
@@ -552,7 +862,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   e1f2g2 <- ggroup(horizontal = TRUE, spacing = 2, container = e1f2)
 
   e1f2_print_interval_chk <- gcheckbox(
-    text = "Print conservative T value",
+    text = strChkPrintTcons,
     checked = TRUE,
     container = e1f2g2
   )
@@ -561,12 +871,12 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   e1f2g3 <- ggroup(horizontal = TRUE, spacing = 2, container = e1f2)
 
   e1f2_mark_interval_chk <- gcheckbox(
-    text = "Draw prediction interval:",
+    text = strChkPredictionInterval,
     checked = TRUE,
     container = e1f2g3
   )
 
-  glabel("Alpha", container = e1f2g3)
+  glabel(text = strLblAlpha, container = e1f2g3)
 
   e1f2_interval_spb <- gspinbutton(
     from = 0, to = 1, by = 0.01,
@@ -574,7 +884,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     container = e1f2g3
   )
 
-  glabel("Fill colour", container = e1f2g3)
+  glabel(text = strLblFill, container = e1f2g3)
   e1f2_interval_drp <- gcombobox(
     items = palette(),
     selected = 2,
@@ -585,7 +895,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # EXPAND 2 ##################################################################
 
   e2 <- gexpandgroup(
-    text = "Data points",
+    text = strExpPoints,
     horizontal = FALSE,
     container = f1
   )
@@ -598,32 +908,32 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   e2g1 <- glayout(container = e2f1)
 
   e2g1[1, 1] <- e2g1_plotpoints_chk <- gcheckbox(
-    text = "Plot data points",
+    text = strChkPoints,
     checked = TRUE,
     container = e2g1
   )
-  e2g1[1, 2] <- glabel(text = "Shape:", container = e2g1)
+  e2g1[1, 2] <- glabel(text = strLblShape, container = e2g1)
   e2g1[1, 3] <- e2g1_shape_spb <- gspinbutton(
     from = 0, to = 25,
     by = 1, value = 18,
     container = e2g1
   )
 
-  e2g1[1, 4] <- glabel(text = "Alpha:", container = e2g1)
+  e2g1[1, 4] <- glabel(text = strLblAlpha, container = e2g1)
   e2g1[1, 5] <- e2g1_alpha_spb <- gspinbutton(
     from = 0, to = 1,
     by = 0.01, value = 0.60,
     container = e2g1
   )
 
-  e2g1[1, 6] <- glabel(text = "Jitter (h/v):", container = e2g1)
+  e2g1[1, 6] <- glabel(text = strLblJitter, container = e2g1)
   e2g1[1, 7] <- e2g1_jitterh_edt <- gedit(text = "0", width = 4, container = e2g1)
   e2g1[1, 8] <- e2g1_jitterv_edt <- gedit(text = "0", width = 4, container = e2g1)
 
   # EXPAND 3 ##################################################################
 
   e3 <- gexpandgroup(
-    text = "Axes",
+    text = strExpAxes,
     horizontal = FALSE,
     container = f1
   )
@@ -633,29 +943,30 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
   e3f1 <- gframe(text = "", horizontal = FALSE, container = e3)
 
-  log10_y_scale_chk <- gcheckbox("Use log10 scale at Y axis",
+  log10_y_scale_chk <- gcheckbox(
+    text = strChkScaleLog,
     checked = FALSE,
     container = e3f1
   )
 
   glabel(
-    text = "NB! Must provide both min and max value.",
+    text = strLblNbMinMax,
     anchor = c(-1, 0), container = e3f1
   )
 
   e3g1 <- glayout(container = e3f1, spacing = 1)
-  e3g1[1, 1:2] <- glabel(text = "Limit Y axis (min-max)", container = e3g1)
+  e3g1[1, 1:2] <- glabel(text = strLblLimitY, container = e3g1)
   e3g1[2, 1] <- e3g1_y_min_edt <- gedit(text = "", width = 5, container = e3g1)
   e3g1[2, 2] <- e3g1_y_max_edt <- gedit(text = "", width = 5, container = e3g1)
 
-  e3g1[3, 1:2] <- glabel(text = "Limit X axis (min-max)", container = e3g1)
+  e3g1[3, 1:2] <- glabel(text = strLblLimitX, container = e3g1)
   e3g1[4, 1] <- e3g1_x_min_edt <- gedit(text = "", width = 5, container = e3g1)
   e3g1[4, 2] <- e3g1_x_max_edt <- gedit(text = "", width = 5, container = e3g1)
 
   # FRAME 4 ###################################################################
 
   e4 <- gexpandgroup(
-    text = "X labels",
+    text = strExpLabels,
     horizontal = FALSE,
     container = f1
   )
@@ -667,17 +978,17 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
   e4g1 <- glayout(container = e4f1)
 
-  e4g1[1, 1] <- glabel(text = "Text size (pts):", container = e4g1)
+  e4g1[1, 1] <- glabel(text = strLblSize, container = e4g1)
   e4g1[1, 2] <- e4g1_size_edt <- gedit(text = "8", width = 4, container = e4g1)
 
-  e4g1[1, 3] <- glabel(text = "Angle:", container = e4g1)
+  e4g1[1, 3] <- glabel(text = strLblAngle, container = e4g1)
   e4g1[1, 4] <- e4g1_angle_spb <- gspinbutton(
     from = 0, to = 360, by = 1,
     value = 0,
     container = e4g1
   )
 
-  e4g1[2, 1] <- glabel(text = "Justification (v/h):", container = e4g1)
+  e4g1[2, 1] <- glabel(text = strLblJustification, container = e4g1)
   e4g1[2, 2] <- e4g1_vjust_spb <- gspinbutton(
     from = 0, to = 1, by = 0.1,
     value = 0.5,
@@ -914,13 +1225,13 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       yTitle <- val_ytitle
     } else {
       if (val_h) {
-        mainTitle <- "Drop-out probability as a function of average peak height"
-        xTitle <- "Average peak height 'H', (RFU)"
+        mainTitle <- strLblMainTitleAverage
+        xTitle <- strLblXTitleAverage
       } else {
-        mainTitle <- "Drop-out probability as a function of present-allele height"
-        xTitle <- "Peak height, (RFU)"
+        mainTitle <- strLblMainTitleHeight
+        xTitle <- strLblXTitleHeight
       }
-      yTitle <- "Drop-out probability, P(D)"
+      yTitle <- strLblYTitle
     }
 
     # Extract model parameters.
@@ -954,7 +1265,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     yplot <- plogis(ypred$fit)
 
     # Create legend text.
-    legendModel <- paste("Model parameters: \u03B20=", round(b0, 3),
+    legendModel <- paste(strLblLegend, " \u03B20=", round(b0, 3),
       ", \u03B21=", round(b1, 3),
       sep = ""
     )
@@ -962,7 +1273,7 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     if (hosOk) {
       # Add Hosmer-Lemeshow test.
       legendModel <- paste(legendModel,
-        "\nHosmer-Lemeshow test: p = ", round(hos$p.value, 4),
+        strLblHosmer, round(hos$p.value, 4),
         sep = ""
       )
     } else {
@@ -1011,7 +1322,8 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       gp <- ggplot(
         data = predictionDf,
         aes_string(y = "Prob", x = "Exp")
-      ) + geom_line()
+      ) +
+        geom_line()
 
       # Plot observed data points (heterozygotes).
       if (val_points) {
@@ -1223,13 +1535,13 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       svalue(f5_save_btn) <- "Save as object"
       enabled(f5_save_btn) <- TRUE
       unblockHandlers(f5_save_btn)
-      
+
       # Store in global variable.
       .gPlot <<- gp
     } else {
       gmessage(
-        msg = "Data frame is NULL or NA!",
-        title = "Error",
+        msg = strMsgNull,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
@@ -1304,15 +1616,13 @@ modelDropout_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       }
 
       if (!is.null(missingCol)) {
-        message <- paste("Dataset is ok for drop-out analysis.\n",
-          "However, additional columns are required for this analysis:\n",
-          paste(missingCol, collapse = "\n"),
-          "\n\nPlease try modeling using another scoring method.",
-          sep = ""
-        )
-
-        gmessage(message,
-          title = "message",
+        gmessage(
+          msg = paste(strMsgIncomplete1,
+            paste(missingCol, collapse = "\n"),
+            strMsgIncomplete2,
+            sep = ""
+          ),
+          title = strMsgTitleIncomplete,
           icon = "info",
           parent = w
         )
