@@ -4,6 +4,8 @@
 
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 13.04.2020: Added language support.
+# 13.04.2020: Implemented function checkDataset.
 # 23.02.2019: Compacted and tweaked gui for tcltk.
 # 11.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 13.07.2017: Fixed issue with button handlers.
@@ -22,8 +24,6 @@
 # 11.11.2015: Added more themes.
 # 08.11.2015: Added new plot options 'Hb vs. Marker' and 'Lb vs. Marker'.
 # 08.11.2015: Added options to plot all data 'Facet per marker and wrap by dye'.
-# 29.08.2015: Added importFrom.
-# 17.04.2015: Included a check for plot facet error caused by all NA's.
 
 #' @title Plot Balance
 #'
@@ -60,7 +60,7 @@
 #'
 #' @return TRUE
 #'
-#' @seealso \url{http://docs.ggplot2.org/current/} for details on plot settings.
+#' @seealso \url{https://ggplot2.tidyverse.org/} for details on plot settings.
 
 plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, parent = NULL) {
 
@@ -69,12 +69,285 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   .gDataName <- NULL
   .gPlot <- NULL
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Plot balance"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset and kit"
+  strLblDataset <- "Balance dataset:"
+  strDrpDataset <- "<Select dataset>"
+  strLblSamples <- "samples"
+  strLblKit <- "and the kit used:"
+  strFrmOptions <- "Options"
+  strChkOverride <- "Override automatic titles"
+  strLblTitlePlot <- "Plot title:"
+  strLblTitleX <- "X title:"
+  strLblTitleY <- "Y title:"
+  strLblTheme <- "Plot theme:"
+  strChkSex <- "Exclude sex markers"
+  strChkLog <- "Log (balance)"
+  strRadNone <- "Do not facet or wrap"
+  strRadWrap <- "Wrap by Dye"
+  strRadFacetWrap <- "Facet by Marker and wrap by Dye"
+  strExpPoints <- "Data points"
+  strLblShape <- "Shape:"
+  strLblAlpha <- "Alpha:"
+  strLblJitter <- "Jitter (width):"
+  strExpAxes <- "Axes"
+  strChkScaleLog <- "Use log10 scale at Y axis"
+  strLblLimitY <- "Limit Y axis (min-max)"
+  strLblLimitX <- "Limit X axis (min-max)"
+  strLblScales <- "Scales:"
+  strExpLabels <- "X labels"
+  strLblSize <- "Text size (pts):"
+  strLblAngle <- "Angle:"
+  strLblJustification <- "Justification (v/h):"
+  strFrmPlot <- "Plot balance data"
+  strBtnHbVsHeight <- "Hb vs. Height"
+  strTipHbVsHeight <- "Plot heterozygote balance by mean marker peak height"
+  strBtnHbVsDelta <- "Hb vs. Delta"
+  strTipHbVsDelta <- "Plot heterozygote balance by allele repeat difference"
+  strBtnHbVsH <- "Hb vs. 'H'"
+  strTipHbVsH <- "Plot heterozygote balance by average profile peak height"
+  strBtnHbVsMarker <- "Hb vs. Marker"
+  strTipHbVsMarker <- "Plot heterozygote balance by marker"
+  strBtnLbVsHeight <- "Lb vs. Height"
+  strTipLbVsHeight <- "Plot locus balance by mean marker peak height"
+  strBtnLbVsH <- "Lb vs. 'H'"
+  strTipLbVsH <- "Plot locus balance by average profile peak height"
+  strBtnLbVsMarker <- "Lb vs. Marker"
+  strTipLbVsMarker <- "Plot locus balance by marker"
+  strBtnProcessing <- "Processing..."
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnSaveObject <- "Save as object"
+  strBtnSaveImage <- "Save as image"
+  strBtnObjectSaved <- "Object saved"
+  strLblMainTitleHb <- "Heterozygous balance"
+  strLblMainTitleLb <- "Locus balance"
+  strLblXTitleMean <- "Mean peak height (RFU)"
+  strLblXTitleDelta <- "Repeat difference"
+  strLblXTitleAverage <- "Average peak height 'H' (RFU)"
+  strLblXTitleLocus <- "Locus"
+  strLblXTitleLocusHeight <- "Locus peak height (RFU)"
+  strLblYTitleLog <- "Log(Ratio)"
+  strLblYTitle <- "Ratio"
+  strMsgNull <- "Data frame is NULL or NA!"
+  strMsgTitleError <- "Error"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDataset"]$value
+    strDrpDataset <- ifelse(is.na(strtmp), strDrpDataset, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strLblKit"]$value
+    strLblKit <- ifelse(is.na(strtmp), strLblKit, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strChkOverride"]$value
+    strChkOverride <- ifelse(is.na(strtmp), strChkOverride, strtmp)
+
+    strtmp <- dtStrings["strLblTitlePlot"]$value
+    strLblTitlePlot <- ifelse(is.na(strtmp), strLblTitlePlot, strtmp)
+
+    strtmp <- dtStrings["strLblTitleX"]$value
+    strLblTitleX <- ifelse(is.na(strtmp), strLblTitleX, strtmp)
+
+    strtmp <- dtStrings["strLblTitleY"]$value
+    strLblTitleY <- ifelse(is.na(strtmp), strLblTitleY, strtmp)
+
+    strtmp <- dtStrings["strLblTheme"]$value
+    strLblTheme <- ifelse(is.na(strtmp), strLblTheme, strtmp)
+
+    strtmp <- dtStrings["strChkSex"]$value
+    strChkSex <- ifelse(is.na(strtmp), strChkSex, strtmp)
+
+    strtmp <- dtStrings["strChkLog"]$value
+    strChkLog <- ifelse(is.na(strtmp), strChkLog, strtmp)
+
+    strtmp <- dtStrings["strRadNone"]$value
+    strRadNone <- ifelse(is.na(strtmp), strRadNone, strtmp)
+
+    strtmp <- dtStrings["strRadWrap"]$value
+    strRadWrap <- ifelse(is.na(strtmp), strRadWrap, strtmp)
+
+    strtmp <- dtStrings["strRadFacetWrap"]$value
+    strRadFacetWrap <- ifelse(is.na(strtmp), strRadFacetWrap, strtmp)
+
+    strtmp <- dtStrings["strExpPoints"]$value
+    strExpPoints <- ifelse(is.na(strtmp), strExpPoints, strtmp)
+
+    strtmp <- dtStrings["strLblShape"]$value
+    strLblShape <- ifelse(is.na(strtmp), strLblShape, strtmp)
+
+    strtmp <- dtStrings["strLblAlpha"]$value
+    strLblAlpha <- ifelse(is.na(strtmp), strLblAlpha, strtmp)
+
+    strtmp <- dtStrings["strLblJitter"]$value
+    strLblJitter <- ifelse(is.na(strtmp), strLblJitter, strtmp)
+
+    strtmp <- dtStrings["strExpAxes"]$value
+    strExpAxes <- ifelse(is.na(strtmp), strExpAxes, strtmp)
+
+    strtmp <- dtStrings["strChkScaleLog"]$value
+    strChkScaleLog <- ifelse(is.na(strtmp), strChkScaleLog, strtmp)
+
+    strtmp <- dtStrings["strLblLimitY"]$value
+    strLblLimitY <- ifelse(is.na(strtmp), strLblLimitY, strtmp)
+
+    strtmp <- dtStrings["strLblLimitX"]$value
+    strLblLimitX <- ifelse(is.na(strtmp), strLblLimitX, strtmp)
+
+    strtmp <- dtStrings["strLblScales"]$value
+    strLblScales <- ifelse(is.na(strtmp), strLblScales, strtmp)
+
+    strtmp <- dtStrings["strExpLabels"]$value
+    strExpLabels <- ifelse(is.na(strtmp), strExpLabels, strtmp)
+
+    strtmp <- dtStrings["strLblSize"]$value
+    strLblSize <- ifelse(is.na(strtmp), strLblSize, strtmp)
+
+    strtmp <- dtStrings["strLblAngle"]$value
+    strLblAngle <- ifelse(is.na(strtmp), strLblAngle, strtmp)
+
+    strtmp <- dtStrings["strLblJustification"]$value
+    strLblJustification <- ifelse(is.na(strtmp), strLblJustification, strtmp)
+
+    strtmp <- dtStrings["strFrmPlot"]$value
+    strFrmPlot <- ifelse(is.na(strtmp), strFrmPlot, strtmp)
+
+    strtmp <- dtStrings["strBtnHbVsHeight"]$value
+    strBtnHbVsHeight <- ifelse(is.na(strtmp), strBtnHbVsHeight, strtmp)
+
+    strtmp <- dtStrings["strTipHbVsHeight"]$value
+    strTipHbVsHeight <- ifelse(is.na(strtmp), strTipHbVsHeight, strtmp)
+
+    strtmp <- dtStrings["strBtnHbVsDelta"]$value
+    strBtnHbVsDelta <- ifelse(is.na(strtmp), strBtnHbVsDelta, strtmp)
+
+    strtmp <- dtStrings["strTipHbVsDelta"]$value
+    strTipHbVsDelta <- ifelse(is.na(strtmp), strTipHbVsDelta, strtmp)
+
+    strtmp <- dtStrings["strBtnHbVsH"]$value
+    strBtnHbVsH <- ifelse(is.na(strtmp), strBtnHbVsH, strtmp)
+
+    strtmp <- dtStrings["strTipHbVsH"]$value
+    strTipHbVsH <- ifelse(is.na(strtmp), strTipHbVsH, strtmp)
+
+    strtmp <- dtStrings["strBtnHbVsMarker"]$value
+    strBtnHbVsMarker <- ifelse(is.na(strtmp), strBtnHbVsMarker, strtmp)
+
+    strtmp <- dtStrings["strTipHbVsMarker"]$value
+    strTipHbVsMarker <- ifelse(is.na(strtmp), strTipHbVsMarker, strtmp)
+
+    strtmp <- dtStrings["strBtnLbVsHeight"]$value
+    strBtnLbVsHeight <- ifelse(is.na(strtmp), strBtnLbVsHeight, strtmp)
+
+    strtmp <- dtStrings["strTipLbVsHeight"]$value
+    strTipLbVsHeight <- ifelse(is.na(strtmp), strTipLbVsHeight, strtmp)
+
+    strtmp <- dtStrings["strBtnLbVsH"]$value
+    strBtnLbVsH <- ifelse(is.na(strtmp), strBtnLbVsH, strtmp)
+
+    strtmp <- dtStrings["strTipLbVsH"]$value
+    strTipLbVsH <- ifelse(is.na(strtmp), strTipLbVsH, strtmp)
+
+    strtmp <- dtStrings["strBtnLbVsMarker"]$value
+    strBtnLbVsMarker <- ifelse(is.na(strtmp), strBtnLbVsMarker, strtmp)
+
+    strtmp <- dtStrings["strTipLbVsMarker"]$value
+    strTipLbVsMarker <- ifelse(is.na(strtmp), strTipLbVsMarker, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnSaveObject"]$value
+    strBtnSaveObject <- ifelse(is.na(strtmp), strBtnSaveObject, strtmp)
+
+    strtmp <- dtStrings["strBtnSaveImage"]$value
+    strBtnSaveImage <- ifelse(is.na(strtmp), strBtnSaveImage, strtmp)
+
+    strtmp <- dtStrings["strBtnObjectSaved"]$value
+    strBtnObjectSaved <- ifelse(is.na(strtmp), strBtnObjectSaved, strtmp)
+
+    strtmp <- dtStrings["strLblMainTitleHb"]$value
+    strLblMainTitleHb <- ifelse(is.na(strtmp), strLblMainTitleHb, strtmp)
+
+    strtmp <- dtStrings["strLblMainTitleLb"]$value
+    strLblMainTitleLb <- ifelse(is.na(strtmp), strLblMainTitleLb, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleMean"]$value
+    strLblXTitleMean <- ifelse(is.na(strtmp), strLblXTitleMean, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleDelta"]$value
+    strLblXTitleDelta <- ifelse(is.na(strtmp), strLblXTitleDelta, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleAverage"]$value
+    strLblXTitleAverage <- ifelse(is.na(strtmp), strLblXTitleAverage, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleLocus"]$value
+    strLblXTitleLocus <- ifelse(is.na(strtmp), strLblXTitleLocus, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleLocusHeight"]$value
+    strLblXTitleLocusHeight <- ifelse(is.na(strtmp), strLblXTitleLocusHeight, strtmp)
+
+    strtmp <- dtStrings["strLblYTitleLog"]$value
+    strLblYTitleLog <- ifelse(is.na(strtmp), strLblYTitleLog, strtmp)
+
+    strtmp <- dtStrings["strLblYTitle"]$value
+    strLblYTitle <- ifelse(is.na(strtmp), strLblYTitle, strtmp)
+
+    strtmp <- dtStrings["strMsgNull"]$value
+    strMsgNull <- ifelse(is.na(strtmp), strMsgNull, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Plot balance", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -117,32 +390,32 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("plotBalance_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Dataset and kit",
+    text = strFrmDataset,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Select dataset:", container = f0)
+  glabel(text = strLblDataset, container = f0)
 
   dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -154,9 +427,12 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
     ellipsize = "none"
   )
 
-  f0_samples_lbl <- glabel(text = " (0 samples)", container = f0)
+  f0_samples_lbl <- glabel(
+    text = paste(" (0 ", strLblSamples, ")", sep = ""),
+    container = f0
+  )
 
-  glabel(text = " and the kit used:", container = f0)
+  glabel(text = strLblKit, container = f0)
 
   kit_drp <- gcombobox(
     items = getKit(),
@@ -188,7 +464,7 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
 
       svalue(f0_samples_lbl) <- paste(" (",
         length(unique(.gData$Sample.Name)),
-        " samples)",
+        " ", strLblSamples, ")",
         sep = ""
       )
 
@@ -209,21 +485,21 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
       .gData <<- NULL
       svalue(f5_save_edt) <- ""
       svalue(dataset_drp, index = TRUE) <- 1
-      svalue(f0_samples_lbl) <- " (0 samples)"
+      svalue(f0_samples_lbl) <- paste(" (0 ", strLblSamples, ")", sep = "")
     }
   })
 
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 2,
     container = gv
   )
 
   titles_chk <- gcheckbox(
-    text = "Override automatic titles.",
+    text = strChkOverride,
     checked = FALSE, container = f1
   )
 
@@ -237,18 +513,18 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   )
 
   # Legends
-  glabel(text = "Plot title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitlePlot, container = titles_group, anchor = c(-1, 0))
   title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
-  glabel(text = "X title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitleX, container = titles_group, anchor = c(-1, 0))
   x_title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
-  glabel(text = "Y title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitleY, container = titles_group, anchor = c(-1, 0))
   y_title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
 
   f1g2 <- glayout(container = f1)
-  f1g2[1, 1] <- glabel(text = "Plot theme:", anchor = c(-1, 0), container = f1g2)
+  f1g2[1, 1] <- glabel(text = strLblTheme, anchor = c(-1, 0), container = f1g2)
   items_theme <- c(
     "theme_grey()", "theme_bw()", "theme_linedraw()",
     "theme_light()", "theme_dark()", "theme_minimal()",
@@ -261,64 +537,59 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
     ellipsize = "none"
   )
 
-  f1_drop_chk <- gcheckbox(text = "Drop sex markers", checked = FALSE, container = f1)
+  f1_drop_chk <- gcheckbox(text = strChkSex, checked = FALSE, container = f1)
 
   f1_logHb_chk <- gcheckbox(
-    text = "Plot Log(balance)", checked = FALSE,
+    text = strChkLog, checked = FALSE,
     container = f1
   )
 
   f1_wrap_opt <- gradio(
-    items = c(
-      "Do not facet or wrap",
-      "Wrap by Dye",
-      "Facet by Marker and wrap by Dye"
-    ),
+    items = c(strRadNone, strRadWrap, strRadFacetWrap),
     horizontal = FALSE, container = f1
   )
 
   # FRAME 7 ###################################################################
 
   f7 <- gframe(
-    text = "Plot Balance data",
+    text = strFrmPlot,
     horizontal = TRUE,
     container = gv,
     spacing = 2
   )
 
-  plot_hb_btn <- gbutton(text = "Hb vs. Height", container = f7)
+  plot_hb_btn <- gbutton(text = strBtnHbVsHeight, container = f7)
+  tooltip(plot_hb_btn) <- strTipHbVsHeight
 
-  plot_hb_d_btn <- gbutton(text = "Hb vs. Delta", container = f7)
+  plot_hb_d_btn <- gbutton(text = strBtnHbVsDelta, container = f7)
+  tooltip(plot_hb_d_btn) <- strTipHbVsDelta
 
-  plot_hb_h_btn <- gbutton(text = "Hb vs. 'H'", container = f7)
+  plot_hb_h_btn <- gbutton(text = strBtnHbVsH, container = f7)
+  tooltip(plot_hb_h_btn) <- strTipHbVsH
 
-  plot_hb_m_btn <- gbutton(text = "Hb vs. Marker", container = f7)
+  plot_hb_m_btn <- gbutton(text = strBtnHbVsMarker, container = f7)
+  tooltip(plot_hb_m_btn) <- strTipHbVsMarker
 
-  plot_lb_btn <- gbutton(text = "Lb vs. Height", container = f7)
+  plot_lb_btn <- gbutton(text = strBtnLbVsHeight, container = f7)
+  tooltip(plot_lb_btn) <- strTipLbVsHeight
 
-  plot_lb_h_btn <- gbutton(text = "Lb vs. 'H'", container = f7)
+  plot_lb_h_btn <- gbutton(text = strBtnLbVsH, container = f7)
+  tooltip(plot_lb_h_btn) <- strTipLbVsH
 
-  plot_lb_m_btn <- gbutton(text = "Lb vs. Marker", container = f7)
+  plot_lb_m_btn <- gbutton(text = strBtnLbVsMarker, container = f7)
+  tooltip(plot_lb_m_btn) <- strTipLbVsMarker
 
   addHandlerChanged(plot_hb_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Hb", "MPH")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_hb_btn) <- FALSE
       .plotBalance(what = "Hb")
       enabled(plot_hb_btn) <- TRUE
@@ -326,24 +597,16 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   })
 
   addHandlerChanged(plot_hb_d_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Delta", "Hb")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_hb_d_btn) <- FALSE
       .plotBalance(what = "Hb_D")
       enabled(plot_hb_d_btn) <- TRUE
@@ -351,24 +614,16 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   })
 
   addHandlerChanged(plot_hb_h_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Hb", "H")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_hb_h_btn) <- FALSE
       .plotBalance(what = "Hb_H")
       enabled(plot_hb_h_btn) <- TRUE
@@ -376,24 +631,16 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   })
 
   addHandlerChanged(plot_hb_m_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Hb")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_hb_m_btn) <- FALSE
       .plotBalance(what = "Hb_M", complex = FALSE)
       enabled(plot_hb_m_btn) <- TRUE
@@ -401,24 +648,16 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   })
 
   addHandlerChanged(plot_lb_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Lb", "TPH")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_lb_btn) <- FALSE
       .plotBalance(what = "Lb")
       enabled(plot_lb_btn) <- TRUE
@@ -426,24 +665,16 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   })
 
   addHandlerChanged(plot_lb_h_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Lb", "H")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_lb_h_btn) <- FALSE
       .plotBalance(what = "Lb_H")
       enabled(plot_lb_h_btn) <- TRUE
@@ -451,24 +682,16 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   })
 
   addHandlerChanged(plot_lb_m_btn, handler = function(h, ...) {
+    val_obj <- svalue(dataset_drp)
 
-    # Check if suitable for plot.
+    # Check if suitable.
     requiredCol <- c("Sample.Name", "Marker", "Lb")
+    ok <- checkDataset(
+      name = val_obj, reqcol = requiredCol,
+      env = env, parent = w, debug = debug
+    )
 
-    if (!all(requiredCol %in% colnames(.gData))) {
-      missingCol <- requiredCol[!requiredCol %in% colnames(.gData)]
-
-      message <- paste("Additional columns required:\n",
-        paste(missingCol, collapse = "\n"),
-        sep = ""
-      )
-
-      gmessage(message,
-        title = "message",
-        icon = "error",
-        parent = w
-      )
-    } else {
+    if (ok) {
       enabled(plot_lb_m_btn) <- FALSE
       .plotBalance(what = "Lb_M", complex = FALSE)
       enabled(plot_lb_m_btn) <- TRUE
@@ -478,26 +701,26 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   # FRAME 5 ###################################################################
 
   f5 <- gframe(
-    text = "Save as",
+    text = strFrmSave,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Name for result:", container = f5)
+  glabel(text = strLblSave, container = f5)
 
   f5_save_edt <- gedit(container = f5, expand = TRUE, fill = TRUE)
 
-  f5_save_btn <- gbutton(text = "Save as object", container = f5)
+  f5_save_btn <- gbutton(text = strBtnSaveObject, container = f5)
 
-  f5_ggsave_btn <- gbutton(text = "Save as image", container = f5)
+  f5_ggsave_btn <- gbutton(text = strBtnSaveImage, container = f5)
 
   addHandlerClicked(f5_save_btn, handler = function(h, ...) {
     val_name <- svalue(f5_save_edt)
 
     # Change button.
     blockHandlers(f5_save_btn)
-    svalue(f5_save_btn) <- "Processing..."
+    svalue(f5_save_btn) <- strBtnProcessing
     unblockHandlers(f5_save_btn)
     enabled(f5_save_btn) <- FALSE
 
@@ -509,7 +732,7 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
 
     # Change button.
     blockHandlers(f5_save_btn)
-    svalue(f5_save_btn) <- "Object saved"
+    svalue(f5_save_btn) <- strBtnObjectSaved
     unblockHandlers(f5_save_btn)
   })
 
@@ -526,7 +749,7 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   # ADVANCED OPTIONS ##########################################################
 
   e2 <- gexpandgroup(
-    text = "Data points",
+    text = strExpPoints,
     horizontal = FALSE,
     container = f1
   )
@@ -536,27 +759,27 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
 
   grid2 <- glayout(container = e2)
 
-  grid2[1, 1] <- glabel(text = "Shape:", container = grid2)
+  grid2[1, 1] <- glabel(text = strLblShape, container = grid2)
   grid2[1, 2] <- e2_shape_spb <- gspinbutton(
     from = 0, to = 25,
     by = 1, value = 18,
     container = grid2
   )
 
-  grid2[1, 3] <- glabel(text = "Alpha:", container = grid2)
+  grid2[1, 3] <- glabel(text = strLblAlpha, container = grid2)
   grid2[1, 4] <- e2_alpha_spb <- gspinbutton(
     from = 0, to = 1,
     by = 0.01, value = 0.60,
     container = grid2
   )
 
-  grid2[1, 5] <- glabel(text = "Jitter (width):", container = grid2)
+  grid2[1, 5] <- glabel(text = strLblJitter, container = grid2)
   grid2[1, 6] <- e2_jitter_edt <- gedit(text = "0", width = 4, container = grid2)
 
   # FRAME 3 ###################################################################
 
   e3 <- gexpandgroup(
-    text = "Axes",
+    text = strExpAxes,
     horizontal = FALSE,
     container = f1
   )
@@ -566,17 +789,17 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
 
   grid3 <- glayout(container = e3, spacing = 1)
 
-  grid3[1, 1:2] <- glabel(text = "Limit Y axis (min-max)", container = grid3)
+  grid3[1, 1:2] <- glabel(text = strLblLimitY, container = grid3)
   grid3[2, 1] <- e3_y_min_edt <- gedit(text = "", width = 5, container = grid3)
   grid3[2, 2] <- e3_y_max_edt <- gedit(text = "", width = 5, container = grid3)
 
-  grid3[3, 1:2] <- glabel(text = "Limit X axis (min-max)", container = grid3)
+  grid3[3, 1:2] <- glabel(text = strLblLimitX, container = grid3)
   grid3[4, 1] <- e3_x_min_edt <- gedit(text = "", width = 5, container = grid3)
   grid3[4, 2] <- e3_x_max_edt <- gedit(text = "", width = 5, container = grid3)
 
   grid3[1, 3] <- glabel(text = "    ", container = grid3) # Add some space.
 
-  grid3[1, 4] <- glabel(text = "Scales:", container = grid3)
+  grid3[1, 4] <- glabel(text = strLblScales, container = grid3)
   grid3[2:4, 4] <- e3_scales_opt <- gradio(
     items = c("fixed", "free_x", "free_y", "free"),
     selected = 1,
@@ -587,7 +810,7 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
   # FRAME 4 ###################################################################
 
   e4 <- gexpandgroup(
-    text = "X labels",
+    text = strExpLabels,
     horizontal = FALSE,
     container = f1
   )
@@ -597,17 +820,17 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
 
   grid4 <- glayout(container = e4)
 
-  grid4[1, 1] <- glabel(text = "Text size (pts):", container = grid4)
+  grid4[1, 1] <- glabel(text = strLblSize, container = grid4)
   grid4[1, 2] <- e4_size_edt <- gedit(text = "10", width = 4, container = grid4)
 
-  grid4[1, 3] <- glabel(text = "Angle:", container = grid4)
+  grid4[1, 3] <- glabel(text = strLblAngle, container = grid4)
   grid4[1, 4] <- e4_angle_spb <- gspinbutton(
     from = 0, to = 360, by = 1,
     value = 270,
     container = grid4
   )
 
-  grid4[2, 1] <- glabel(text = "Justification (v/h):", container = grid4)
+  grid4[2, 1] <- glabel(text = strLblJustification, container = grid4)
   grid4[2, 2] <- e4_vjust_spb <- gspinbutton(
     from = 0, to = 1, by = 0.1,
     value = 0.5,
@@ -620,11 +843,8 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
     container = grid4
   )
 
-
-
   # FUNCTIONS #################################################################
 
-
   .plotBalance <- function(what, complex = NULL) {
 
     # Get values.
@@ -809,60 +1029,60 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
         }
 
         if (what == "Hb") {
-          mainTitle <- "Heterozygous balance"
-          xTitle <- "Mean peak height (RFU)"
+          mainTitle <- strLblMainTitleHb
+          xTitle <- strLblXTitleMean
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else if (what == "Hb_D") {
-          mainTitle <- "Heterozygous balance"
-          xTitle <- "Repeat difference"
+          mainTitle <- strLblMainTitleHb
+          xTitle <- strLblXTitleDelta
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else if (what == "Hb_H") {
-          mainTitle <- "Heterozygous balance"
-          xTitle <- "Average peak height 'H' (RFU)"
+          mainTitle <- strLblMainTitleHb
+          xTitle <- strLblXTitleAverage
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else if (what == "Hb_M") {
-          mainTitle <- "Heterozygous balance"
-          xTitle <- "Locus"
+          mainTitle <- strLblMainTitleHb
+          xTitle <- strLblXTitleLocus
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else if (what == "Lb") {
-          mainTitle <- "Locus balance"
-          xTitle <- "Locus peak height (RFU)"
+          mainTitle <- strLblMainTitleLb
+          xTitle <- strLblXTitleLocusHeight
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else if (what == "Lb_H") {
-          mainTitle <- "Locus balance"
-          xTitle <- "Average peak height 'H' (RFU)"
+          mainTitle <- strLblMainTitleLb
+          xTitle <- strLblXTitleAverage
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else if (what == "Lb_M") {
-          mainTitle <- "Locus balance"
-          xTitle <- "Locus"
+          mainTitle <- strLblMainTitleLb
+          xTitle <- strLblXTitleLocus
           if (val_log) {
-            yTitle <- "Log(Ratio)"
+            yTitle <- strLblYTitleLog
           } else {
-            yTitle <- "Ratio"
+            yTitle <- strLblYTitle
           }
         } else {
           stop(paste("what =", what, "not implemented for create title!"))
@@ -1038,7 +1258,7 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
         print(gp)
 
         # Change save button.
-        svalue(f5_save_btn) <- "Save as object"
+        svalue(f5_save_btn) <- strBtnSaveObject
         enabled(f5_save_btn) <- TRUE
       } else if (complex) {
         # Complex plot, unequal number of markers per dye.
@@ -1261,7 +1481,7 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
         gp <- gridExtra::arrangeGrob(g)
 
         # Change save button.
-        svalue(f5_save_btn) <- "Save as object"
+        svalue(f5_save_btn) <- strBtnSaveObject
         enabled(f5_save_btn) <- FALSE
       } # End if(complex)
 
@@ -1269,8 +1489,8 @@ plotBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE,
       .gPlot <<- gp
     } else {
       gmessage(
-        msg = "Data frame is NULL or NA!",
-        title = "Error",
+        msg = strMsgNull,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
