@@ -1,5 +1,6 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 02.05.2020: Added language support.
 # 24.02.2019: Compacted and tweaked gui for tcltk.
 # 17.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
 # 18.07.2017: Fixed 'ymax' warning (removed) and label order (added 'rev').
@@ -60,12 +61,157 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   # between classes. Qualitative schemes are best suited to representing
   # nominal or categorical data.
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Plot result type"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset"
+  strLblDataset <- "Result type dataset:"
+  strDrpDataset <- "<Select dataset>"
+  strLblSamples <- "samples"
+  strFrmOptions <- "Options"
+  strLblScales <- "Scales:"
+  strChkOverride <- "Override automatic titles"
+  strLblTitlePlot <- "Plot title:"
+  strLblTitleX <- "X title:"
+  strLblTitleY <- "Y title:"
+  strChkProportion <- "Plot proportion"
+  strLblBaseSize <- "Base font size (pts):"
+  strLblPalette <- "Color palette:"
+  strChkLabels <- "Print values as bar labels"
+  strLblLabelDecimals <- "Bar label decimals:"
+  strLblLabelSize <- "Bar label font size (pts):"
+  strBtnPlot <- "Plot"
+  strBtnProcessing <- "Processing..."
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnSaveObject <- "Save as object"
+  strBtnSaveImage <- "Save as image"
+  strBtnObjectSaved <- "Object saved"
+  strLblMainTitle <- "samples analysed"
+  strLblXTitleType <- "Result type"
+  strLblYTitleProportion <- "Proportion"
+  strLblYTitleCount <- "Count"
+  strMsgNull <- "Data frame is NULL or NA!"
+  strMsgTitleError <- "Error"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDataset"]$value
+    strDrpDataset <- ifelse(is.na(strtmp), strDrpDataset, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strLblScales"]$value
+    strLblScales <- ifelse(is.na(strtmp), strLblScales, strtmp)
+
+    strtmp <- dtStrings["strChkOverride"]$value
+    strChkOverride <- ifelse(is.na(strtmp), strChkOverride, strtmp)
+
+    strtmp <- dtStrings["strLblTitlePlot"]$value
+    strLblTitlePlot <- ifelse(is.na(strtmp), strLblTitlePlot, strtmp)
+
+    strtmp <- dtStrings["strLblTitleX"]$value
+    strLblTitleX <- ifelse(is.na(strtmp), strLblTitleX, strtmp)
+
+    strtmp <- dtStrings["strLblTitleY"]$value
+    strLblTitleY <- ifelse(is.na(strtmp), strLblTitleY, strtmp)
+
+    strtmp <- dtStrings["strChkProportion"]$value
+    strChkProportion <- ifelse(is.na(strtmp), strChkProportion, strtmp)
+
+    strtmp <- dtStrings["strLblBaseSize"]$value
+    strLblBaseSize <- ifelse(is.na(strtmp), strLblBaseSize, strtmp)
+
+    strtmp <- dtStrings["strLblPalette"]$value
+    strLblPalette <- ifelse(is.na(strtmp), strLblPalette, strtmp)
+
+    strtmp <- dtStrings["strChkLabels"]$value
+    strChkLabels <- ifelse(is.na(strtmp), strChkLabels, strtmp)
+
+    strtmp <- dtStrings["strLblLabelDecimals	"]$value
+    strLblLabelDecimals <- ifelse(is.na(strtmp), strLblLabelDecimals, strtmp)
+
+    strtmp <- dtStrings["strLblLabelSize	"]$value
+    strLblLabelSize <- ifelse(is.na(strtmp), strLblLabelSize, strtmp)
+
+    strtmp <- dtStrings["strBtnPlot"]$value
+    strBtnPlot <- ifelse(is.na(strtmp), strBtnPlot, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnSaveObject"]$value
+    strBtnSaveObject <- ifelse(is.na(strtmp), strBtnSaveObject, strtmp)
+
+    strtmp <- dtStrings["strBtnSaveImage"]$value
+    strBtnSaveImage <- ifelse(is.na(strtmp), strBtnSaveImage, strtmp)
+
+    strtmp <- dtStrings["strBtnObjectSaved"]$value
+    strBtnObjectSaved <- ifelse(is.na(strtmp), strBtnObjectSaved, strtmp)
+
+    strtmp <- dtStrings["strLblMainTitle"]$value
+    strLblMainTitle <- ifelse(is.na(strtmp), strLblMainTitle, strtmp)
+
+    strtmp <- dtStrings["strLblXTitleType"]$value
+    strLblXTitleType <- ifelse(is.na(strtmp), strLblXTitleType, strtmp)
+
+    strtmp <- dtStrings["strLblYTitleProportion"]$value
+    strLblYTitleProportion <- ifelse(is.na(strtmp), strLblYTitleProportion, strtmp)
+
+    strtmp <- dtStrings["strLblYTitleCount"]$value
+    strLblYTitleCount <- ifelse(is.na(strtmp), strLblYTitleCount, strtmp)
+
+    strtmp <- dtStrings["strMsgNull"]$value
+    strMsgNull <- ifelse(is.na(strtmp), strMsgNull, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Plot result type", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -108,32 +254,32 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("plotResultType_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Dataset",
+    text = strFrmDataset,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Select dataset:", container = f0)
+  glabel(text = strLblDataset, container = f0)
 
   dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -145,7 +291,10 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
     ellipsize = "none"
   )
 
-  f0_samples_lbl <- glabel(text = " (0 samples)", container = f0)
+  f0_samples_lbl <- glabel(
+    text = paste(" (0 ", strLblSamples, ")", sep = ""),
+    container = f0
+  )
 
   addHandlerChanged(dataset_drp, handler = function(h, ...) {
     val_obj <- svalue(dataset_drp)
@@ -168,7 +317,7 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
       svalue(f0_samples_lbl) <- paste(" (",
         nrow(.gData),
-        " samples)",
+        " ", strLblSamples, ")",
         sep = ""
       )
 
@@ -180,21 +329,21 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       .gData <<- NULL
       svalue(f5_save_edt) <- ""
       svalue(dataset_drp, index = TRUE) <- 1
-      svalue(f0_samples_lbl) <- " (0 samples)"
+      svalue(f0_samples_lbl) <- paste(" (0 ", strLblSamples, ")", sep = "")
     }
   })
 
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 2,
     container = gv
   )
 
   titles_chk <- gcheckbox(
-    text = "Override automatic titles.",
+    text = strChkOverride,
     checked = FALSE, container = f1
   )
 
@@ -209,28 +358,28 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   )
 
   # Legends
-  glabel(text = "Plot title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitlePlot, container = titles_group, anchor = c(-1, 0))
   title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
-  glabel(text = "X title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitleX, container = titles_group, anchor = c(-1, 0))
   x_title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
-  glabel(text = "Y title:", container = titles_group, anchor = c(-1, 0))
+  glabel(text = strLblTitleY, container = titles_group, anchor = c(-1, 0))
   y_title_edt <- gedit(expand = TRUE, fill = TRUE, container = titles_group)
 
 
   f1_prop_chk <- gcheckbox(
-    text = "Plot proportion",
+    text = strChkProportion,
     checked = TRUE,
     container = f1
   )
 
   grid2 <- glayout(container = f1, spacing = 1)
-  grid2[1, 1] <- glabel(text = "Base font size (pts):", container = grid2)
+  grid2[1, 1] <- glabel(text = strLblBaseSize, container = grid2)
   grid2[1, 2] <- f1_base_size_edt <- gedit(text = "18", width = 4, container = grid2)
 
   grid3 <- glayout(container = f1, spacing = 1)
-  grid3[1, 1] <- glabel(text = "Color palette:", container = grid3)
+  grid3[1, 1] <- glabel(text = strLblPalette, container = grid3)
   grid3[1, 2] <- f1_palette_drp <- gcombobox(
     items = .palette,
     selected = 1,
@@ -240,19 +389,19 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   )
 
   grid4 <- glayout(container = f1, spacing = 1)
-  grid4[1, 1] <- f1_print_chk <- gcheckbox(text = "Print values as bar labels", checked = TRUE, container = grid4)
-  grid4[2, 1] <- glabel(text = "Number of decimals for bar labels:", container = grid4)
+  grid4[1, 1] <- f1_print_chk <- gcheckbox(text = strChkLabels, checked = TRUE, container = grid4)
+  grid4[2, 1] <- glabel(text = strLblLabelDecimals, container = grid4)
   grid4[2, 2] <- f1_decimal_spb <- gspinbutton(
     from = 0, to = 9, by = 1, value = 4,
     container = grid4
   )
-  grid4[3, 1] <- glabel(text = "Font size for bar labels (pts):", container = grid4)
+  grid4[3, 1] <- glabel(text = strLblLabelSize, container = grid4)
   grid4[3, 2] <- f1_lab_size_edt <- gedit(text = "5", width = 4, container = grid4)
 
 
   # FRAME 7 ###################################################################
 
-  plot_btn <- gbutton(text = "Plot", container = gv)
+  plot_btn <- gbutton(text = strBtnPlot, container = gv)
 
   addHandlerChanged(plot_btn, handler = function(h, ...) {
     enabled(plot_btn) <- FALSE
@@ -264,26 +413,26 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
   # FRAME 5 ###################################################################
 
   f5 <- gframe(
-    text = "Save as",
+    text = strFrmSave,
     horizontal = TRUE,
     spacing = 2,
     container = gv
   )
 
-  glabel(text = "Name for result:", container = f5)
+  glabel(text = strLblSave, container = f5)
 
   f5_save_edt <- gedit(container = f5, expand = TRUE, fill = TRUE)
 
-  f5_save_btn <- gbutton(text = "Save as object", container = f5)
+  f5_save_btn <- gbutton(text = strBtnSaveObject, container = f5)
 
-  f5_ggsave_btn <- gbutton(text = "Save as image", container = f5)
+  f5_ggsave_btn <- gbutton(text = strBtnSaveImage, container = f5)
 
   addHandlerClicked(f5_save_btn, handler = function(h, ...) {
     val_name <- svalue(f5_save_edt)
 
     # Change button.
     blockHandlers(f5_save_btn)
-    svalue(f5_save_btn) <- "Processing..."
+    svalue(f5_save_btn) <- strBtnProcessing
     unblockHandlers(f5_save_btn)
     enabled(f5_save_btn) <- FALSE
 
@@ -295,7 +444,7 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
 
     # Change button.
     blockHandlers(f5_save_btn)
-    svalue(f5_save_btn) <- "Object saved"
+    svalue(f5_save_btn) <- strBtnObjectSaved
     unblockHandlers(f5_save_btn)
   })
 
@@ -374,12 +523,12 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
         xTitle <- val_x_title
         yTitle <- val_y_title
       } else {
-        mainTitle <- paste("Analysis of", numberOfSamples, "samples")
-        xTitle <- "Result type"
+        mainTitle <- paste(numberOfSamples, strLblMainTitle)
+        xTitle <- strLblXTitleType
         if (val_prop) {
-          yTitle <- "Proportion"
+          yTitle <- strLblYTitleProportion
         } else {
-          yTitle <- "Count"
+          yTitle <- strLblYTitleCount
         }
       }
 
@@ -437,12 +586,12 @@ plotResultType_gui <- function(env = parent.frame(), savegui = NULL, debug = FAL
       .gPlot <<- gp
 
       # Change save button.
-      svalue(f5_save_btn) <- "Save as object"
+      svalue(f5_save_btn) <- strBtnSaveObject
       enabled(f5_save_btn) <- TRUE
     } else {
       gmessage(
-        msg = "Data frame is NULL or NA!",
-        title = "Error",
+        msg = strMsgNull,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
