@@ -1,5 +1,7 @@
 ################################################################################
 # CHANGE LOG (last 20 changes)
+# 07.06.2020: Added .Deprecated("calculateStatistics_gui")
+# 03.05.2020: Added language support.
 # 15.02.2019: Expand text fields in tcltk by setting fill = TRUE.
 # 11.02.2019: Minor adjustments to tcltk gui.
 # 11.02.2019: Fixed Error in if (svalue(savegui_chk)) { : argument is of length zero (tcltk)
@@ -18,8 +20,6 @@
 # 11.10.2014: Added 'focus', added 'parent' parameter.
 # 28.06.2014: Added help button and moved save gui checkbox.
 # 08.05.2014: Implemented 'checkDataset'.
-# 05.05.2014: Changed default value for percentile 0.95 -> 0.05.
-# 15.02.2014: First version.
 
 
 #' @title Table Balance
@@ -45,17 +45,107 @@
 #' @seealso \code{\link{tableBalance}}
 
 tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE, parent = NULL) {
+  .Deprecated("calculateStatistics_gui")
 
   # Global variables.
   .gData <- NULL
   .gDataName <- NULL
 
+  # Language ------------------------------------------------------------------
+
+  # Get this functions name from call.
+  fnc <- as.character(match.call()[[1]])
+
   if (debug) {
-    print(paste("IN:", match.call()[[1]]))
+    print(paste("IN:", fnc))
   }
 
+  # Default strings.
+  strWinTitle <- "Calculate summary statistics for balance"
+  strChkGui <- "Save GUI settings"
+  strBtnHelp <- "Help"
+  strFrmDataset <- "Dataset"
+  strLblDataset <- "Dataset:"
+  strDrpDataset <- "<Select dataset>"
+  strLblSamples <- "samples"
+  strFrmOptions <- "Options"
+  strLblQuantile <- "Calculate quantile"
+  strLblBy <- "Statistics to calculate"
+  strRadGlobal <- "global"
+  strRadMarker <- "marker"
+  strFrmSave <- "Save as"
+  strLblSave <- "Name for result:"
+  strBtnCalculate <- "Calculate"
+  strBtnProcessing <- "Processing..."
+  strMsgNull <- "Data frame is NULL or NA!"
+  strMsgTitleError <- "Error"
+
+  # Get strings from language file.
+  dtStrings <- getStrings(gui = fnc)
+
+  # If language file is found.
+  if (!is.null(dtStrings)) {
+    # Get language strings, use default if not found.
+
+    strtmp <- dtStrings["strWinTitle"]$value
+    strWinTitle <- ifelse(is.na(strtmp), strWinTitle, strtmp)
+
+    strtmp <- dtStrings["strChkGui"]$value
+    strChkGui <- ifelse(is.na(strtmp), strChkGui, strtmp)
+
+    strtmp <- dtStrings["strBtnHelp"]$value
+    strBtnHelp <- ifelse(is.na(strtmp), strBtnHelp, strtmp)
+
+    strtmp <- dtStrings["strFrmDataset"]$value
+    strFrmDataset <- ifelse(is.na(strtmp), strFrmDataset, strtmp)
+
+    strtmp <- dtStrings["strLblDataset"]$value
+    strLblDataset <- ifelse(is.na(strtmp), strLblDataset, strtmp)
+
+    strtmp <- dtStrings["strDrpDataset"]$value
+    strDrpDataset <- ifelse(is.na(strtmp), strDrpDataset, strtmp)
+
+    strtmp <- dtStrings["strLblSamples"]$value
+    strLblSamples <- ifelse(is.na(strtmp), strLblSamples, strtmp)
+
+    strtmp <- dtStrings["strFrmOptions"]$value
+    strFrmOptions <- ifelse(is.na(strtmp), strFrmOptions, strtmp)
+
+    strtmp <- dtStrings["strLblQuantile"]$value
+    strLblQuantile <- ifelse(is.na(strtmp), strLblQuantile, strtmp)
+
+    strtmp <- dtStrings["strLblBy"]$value
+    strLblBy <- ifelse(is.na(strtmp), strLblBy, strtmp)
+
+    strtmp <- dtStrings["strRadGlobal"]$value
+    strRadGlobal <- ifelse(is.na(strtmp), strRadGlobal, strtmp)
+
+    strtmp <- dtStrings["strRadMarker"]$value
+    strRadMarker <- ifelse(is.na(strtmp), strRadMarker, strtmp)
+
+    strtmp <- dtStrings["strFrmSave"]$value
+    strFrmSave <- ifelse(is.na(strtmp), strFrmSave, strtmp)
+
+    strtmp <- dtStrings["strLblSave"]$value
+    strLblSave <- ifelse(is.na(strtmp), strLblSave, strtmp)
+
+    strtmp <- dtStrings["strBtnCalculate"]$value
+    strBtnCalculate <- ifelse(is.na(strtmp), strBtnCalculate, strtmp)
+
+    strtmp <- dtStrings["strBtnProcessing"]$value
+    strBtnProcessing <- ifelse(is.na(strtmp), strBtnProcessing, strtmp)
+
+    strtmp <- dtStrings["strMsgNull"]$value
+    strMsgNull <- ifelse(is.na(strtmp), strMsgNull, strtmp)
+
+    strtmp <- dtStrings["strMsgTitleError"]$value
+    strMsgTitleError <- ifelse(is.na(strtmp), strMsgTitleError, strtmp)
+  }
+
+  # WINDOW ####################################################################
+
   # Main window.
-  w <- gwindow(title = "Make balance table", visible = FALSE)
+  w <- gwindow(title = strWinTitle, visible = FALSE)
 
   # Runs when window is closed.
   addHandlerUnrealize(w, handler = function(h, ...) {
@@ -99,22 +189,22 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # Help button group.
   gh <- ggroup(container = gv, expand = FALSE, fill = "both")
 
-  savegui_chk <- gcheckbox(text = "Save GUI settings", checked = FALSE, container = gh)
+  savegui_chk <- gcheckbox(text = strChkGui, checked = FALSE, container = gh)
 
   addSpring(gh)
 
-  help_btn <- gbutton(text = "Help", container = gh)
+  help_btn <- gbutton(text = strBtnHelp, container = gh)
 
   addHandlerChanged(help_btn, handler = function(h, ...) {
 
     # Open help page for function.
-    print(help("tableBalance_gui", help_type = "html"))
+    print(help(fnc, help_type = "html"))
   })
 
   # FRAME 0 ###################################################################
 
   f0 <- gframe(
-    text = "Datasets",
+    text = strFrmDataset,
     horizontal = FALSE,
     spacing = 5,
     container = gv
@@ -122,11 +212,11 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
   f0g0 <- glayout(container = f0, spacing = 1)
 
-  f0g0[1, 1] <- glabel(text = "Select dataset:", container = f0g0)
+  f0g0[1, 1] <- glabel(text = strLblDataset, container = f0g0)
 
   f0g0[1, 2] <- f0g0_dataset_drp <- gcombobox(
     items = c(
-      "<Select dataset>",
+      strDrpDataset,
       listObjects(
         env = env,
         obj.class = "data.frame"
@@ -139,7 +229,7 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   )
 
   f0g0[1, 3] <- f0g0_samples_lbl <- glabel(
-    text = " 0 samples",
+    text = paste(" 0", strLblSamples),
     container = f0g0
   )
 
@@ -159,9 +249,9 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       .gData <<- get(val_obj, envir = env)
       .gDataName <<- val_obj
       samples <- length(unique(.gData$Sample.Name))
-      svalue(f0g0_samples_lbl) <- paste(" ", samples, "samples")
-      svalue(f2_save_edt) <- paste(.gDataName,
-        "_table_",
+      svalue(f0g0_samples_lbl) <- paste(" ", samples, strLblSamples)
+      svalue(save_edt) <- paste(.gDataName,
+        "_stat_",
         svalue(f1g1_scope_opt),
         sep = ""
       )
@@ -170,8 +260,8 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       # Reset components.
       .gData <<- data.frame(No.Data = NA)
       .gDataName <<- NULL
-      svalue(f0g0_samples_lbl) <- " 0 samples"
-      svalue(f2_save_edt) <- ""
+      svalue(f0g0_samples_lbl) <- paste(" 0", strLblSamples)
+      svalue(save_edt) <- ""
       svalue(f0g0_dataset_drp, index = TRUE) <- 1
     }
   })
@@ -179,7 +269,7 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
   # FRAME 1 ###################################################################
 
   f1 <- gframe(
-    text = "Options",
+    text = strFrmOptions,
     horizontal = FALSE,
     spacing = 5,
     container = gv
@@ -187,7 +277,7 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
   f1g1 <- glayout(container = f1, spacing = 5)
 
-  f1g1[1, 1] <- glabel(text = "Calculate quantile", container = f1g1)
+  f1g1[1, 1] <- glabel(text = strLblQuantile, container = f1g1)
 
   f1g1[1, 2] <- f1g1_quant_spb <- gspinbutton(
     from = 0, to = 1,
@@ -195,42 +285,33 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     container = f1g1
   )
 
-  f1g1[2, 1] <- glabel(text = "Summarize by", container = f1g1)
+  f1g1[2, 1] <- glabel(text = strLblBy, container = f1g1)
 
   f1g1[3, 1] <- f1g1_scope_opt <- gradio(
-    items = c("global", "locus"),
+    items = c(strRadGlobal, strRadMarker),
     selected = 2,
     horizontal = FALSE,
     container = f1g1
   )
 
   addHandlerChanged(f1g1_scope_opt, handler = function(h, ...) {
-    svalue(f2_save_edt) <- paste(.gDataName, "_table_",
+    svalue(save_edt) <- paste(.gDataName, "_stat_",
       svalue(f1g1_scope_opt),
       sep = ""
     )
   })
 
-  # FRAME 2 ###################################################################
+  # SAVE ######################################################################
 
-  f2 <- gframe(
-    text = "Save as",
-    horizontal = TRUE,
-    spacing = 5,
-    container = gv
-  )
+  save_frame <- gframe(text = strFrmSave, container = gv)
 
-  glabel(text = "Name for result:", container = f2)
+  glabel(text = strLblSave, container = save_frame)
 
-  f2_save_edt <- gedit(text = "", container = f2, expand = TRUE, fill = TRUE)
+  save_edt <- gedit(expand = TRUE, fill = TRUE, container = save_frame)
 
   # BUTTON ####################################################################
 
-  if (debug) {
-    print("BUTTON")
-  }
-
-  run_btn <- gbutton(text = "Summarize", container = gv)
+  run_btn <- gbutton(text = strBtnCalculate, container = gv)
 
   addHandlerClicked(run_btn, handler = function(h, ...) {
 
@@ -239,14 +320,14 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
     val_data_name <- .gDataName
     val_ratio <- as.numeric(svalue(f1g1_quant_spb))
     val_scope <- svalue(f1g1_scope_opt)
-    val_name <- svalue(f2_save_edt)
+    val_name <- svalue(save_edt)
     val_kit <- attr(x = .gData, which = "kit", exact = TRUE)
 
     if (!is.null(.gData)) {
 
       # Change button.
       blockHandlers(run_btn)
-      svalue(run_btn) <- "Processing..."
+      svalue(run_btn) <- strBtnProcessing
       unblockHandlers(run_btn)
       enabled(run_btn) <- FALSE
 
@@ -267,7 +348,7 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       # Update audit trail.
       datanew <- auditTrail(
         obj = datanew, key = keys, value = values,
-        label = "tableBalance_gui", arguments = FALSE,
+        label = fnc, arguments = FALSE,
         package = "strvalidator"
       )
 
@@ -276,7 +357,7 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
 
       if (debug) {
         print(datanew)
-        print(paste("EXIT:", match.call()[[1]]))
+        print(paste("EXIT:", fnc))
       }
 
       # Close GUI.
@@ -284,9 +365,8 @@ tableBalance_gui <- function(env = parent.frame(), savegui = NULL, debug = FALSE
       dispose(w)
     } else {
       gmessage(
-        msg = "Data frame is NULL!\n\n
-               Make sure to select a dataset and a reference set",
-        title = "Error",
+        msg = strMsgNull,
+        title = strMsgTitleError,
         icon = "error"
       )
     }
